/*
* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
*
*    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include <stdlib.h>
#include "pruicss.h"
#include "hw_types.h"
#include "gpio.h"
#include "console_utils.h"
#include "osdrv_osal.h"
#include "osdrv_pruss.h"
#include "hw_icss.h"
#include "hw_icss_pru_ctrl.h"
#include "hw_icss_cfg.h"
#include "pruicss_v1.h"
#include "am437x.h"
#include "hw_control_am43xx.h"
#include "hw_cm_dpll.h"
#include "hw_cm_wkup.h"
#include "board_gpio.h"
#include "hw_icss_intc.h"
#include "hw_icss_iep.h"
#include "hw_edma_tpcc.h"
#include "hw_pwmss_ecap.h"
#include "timpe_pruss_intc_mapping.h"
#include "mpe_common.h"
#include "biss.h"

#include "biss_master_bin.h"

extern PRUICSS_Handle pruIcss0Handle;
extern int mpe_protocol_type;

/**
 * @brief Function called by MPEncoderTask() to establish communciation with Encoder
 *
 *
 */
void MPEncoderBiSS(void)
{
	unsigned int biss_angle;
	float	angle_float;

	// stop PRU firmware
	PRUICSS_pruDisable(pruIcss0Handle, 0);
	PRUICSS_pruReset(pruIcss0Handle, 0);

	// configure clock
	conf_200MHZ();

    // Set C28 to 0
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0) +  HW_ICSS_PRU_CTRL_CTPPR0, ((0x0000<<16)| (0)));

	// configure external pinmux
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MMC0_DAT3), (CONTROL_CONF_MUXMODE(7)));	// gpio2_26 / ENDAT_TXEN

    // configure PRU-ICSS internal pinmux
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + HW_ICSS_CFG_GPCFG0,  0x0<<26);

    // configure GPIO for protocols
    GPIOPinWrite(SOC_GPIO2_REG, 12, GPIO_PIN_LOW);
    GPIOPinWrite(SOC_GPIO1_REG, 13, GPIO_PIN_LOW);
    GPIOPinWrite(SOC_GPIO1_REG, 14, GPIO_PIN_LOW);

	// load firmware
	PRUICSS_setPRUBuffer(pruIcss0Handle, 0, (UInt32*)BiSSFirmware, sizeof(BiSSFirmware));

    // configure shared memory interface
	// clear ICSS0 PRU0 data RAM
	memset((void *)(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x0000), 0, (4*1024));
	// BiSS encoder configuration:	clock_speed=1; crc_bits=8, encoder_bits=12
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x0000,  0x0001080C);

	//Run firmware
	PRUICSS_pruExecProgram(pruIcss0Handle, 0);

	TaskOSAL_sleep(25);
	// write known patterin into position regigister
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x8,  0xFFFFFFFF);
	TaskOSAL_sleep(100);
	// check if BiSS encoder has been detected (pattern has been overwritten by PRU firmware)
	biss_angle = HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x8);
	if(biss_angle == 0xFFFFFFFF)
		return;

	mpe_protocol_type = mpe_endat;
	CONSOLEUtilsPrintf("\nBiSS C\n");

	while(1)
	{
		biss_angle = HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x8);
		angle_float = ((float) biss_angle) / 4096 * 360;

		// simplified method to detect that the decoder is disconnected (write known pattern)
		HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x8,  0xFFFFFFFF);

		if(biss_angle == 0xFFFFFFFF)
		{
			CONSOLEUtilsPrintf("\n");
			return;
		}

		CONSOLEUtilsPrintf("angle=%f\r", angle_float);

		TaskOSAL_sleep(50);
	}

}

