/*
* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
*
*    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include <string.h>
#include "endat.h"
#include "endat_drv.h"
#include "pruicss.h"
#include "hw_types.h"
#include "gpio.h"
#include "console_utils.h"
#include "osdrv_osal.h"
#include "hw_icss.h"
#include "hw_icss_pru_ctrl.h"
#include "hw_icss_cfg.h"
#include "pruicss_v1.h"
#include "am437x.h"
#include "hw_control_am43xx.h"
#include "hw_cm_dpll.h"
#include "mpe_common.h"

#include "endat_master_bin.h"


extern PRUICSS_Handle pruIcss0Handle;
extern int mpe_protocol_type;

static int endat_calc_clock(unsigned freq, struct endat_clk_cfg *clk_cfg);
static void endat_process_host_command(int cmd, struct cmd_supplement *cmd_supplement, struct endat_priv *priv);
static unsigned endat_do_sanity_tst_delay(unsigned delay);
int mpe_endat_wait_initialization(struct endat_priv *priv, unsigned timeout);
static union endat_format_data endat_format_data_mtrctrl;
union position {
	float angle;
	unsigned long long length;
};

#define ENDAT_INPUT_CLOCK_UART_FREQUENCY 192000000
#define ENDAT_INPUT_CLOCK_FREQUENCY ENDAT_INPUT_CLOCK_UART_FREQUENCY
#define ENDAT_RX_SAMPLE_SIZE	7
#define ENDAT_RX_SESQUI_DIV	(1 << 15)

struct endat_priv *priv;

/**
 * @brief Function called by MPEncoderTask() to establish communciation with Encoder
 *
 *
 */
void MPEncoderEnDat22(void)
{
	int i;
	struct cmd_supplement cmd_supplement;

	// stop PRU firmware and reset PRU
	PRUICSS_pruDisable(pruIcss0Handle, 0);
    PRUICSS_pruReset(pruIcss0Handle, 0);

	// configure clock
	conf_200MHZ();

    // Set C28 to 0
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0) +  HW_ICSS_PRU_CTRL_CTPPR0, ((0x0000<<16)| (0)));

	// configure external pinmux
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MMC0_DAT3), (CONTROL_CONF_MUXMODE(5)));	// gpio2_26 / ENDAT_TXEN
    // Stop EnDAT TX: Tx_global_reinit R31.t19
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_DEBUG(0) + (31*4),  1<<19);
    // RX_disable R30
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_DEBUG(0) + (30*4),  0);


	// configure internal pinmux
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + HW_ICSS_CFG_GPCFG0,  0x1<<26);

	// clear ICSS0 PRU0 data RAM
	memset((void *)(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x0000), 0, (4*1024));

	// multiplexer adapter configuration
    GPIOPinWrite(SOC_GPIO2_REG, 12, GPIO_PIN_LOW);
    GPIOPinWrite(SOC_GPIO1_REG, 13, GPIO_PIN_LOW);
    GPIOPinWrite(SOC_GPIO1_REG, 14, GPIO_PIN_HIGH);

    // configure SCU
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDRXCFG,  0x00020007);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDTXCFG,  0x00170700);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH0CFG0,  0x10000000);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH0CFG1,  0x00FA07D0);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH1CFG0,  0x10000000);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH1CFG1,  0x00FA07D0);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH2CFG0,  0x00293800);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH2CFG1,  0x00FA07D0);

	// configure shared memory interface
    // EnDAT:
    priv = endat_init((struct endat_pruss_xchg *) ((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr);
	endat_config_host_trigger(priv);
	endat_config_channel(priv, 2);

	// load firmware
	PRUICSS_setPRUBuffer(pruIcss0Handle, 0, (unsigned int*)EnDatFirmware, sizeof(EnDatFirmware));
	//Run firmware
	PRUICSS_pruExecProgram(pruIcss0Handle, 0);

	/* check initialization ack from firmware, with a timeout of 5 second */
	i = mpe_endat_wait_initialization(priv, 500);

	if (i < 0) {
		return;
	}

	// detected EnDAT
	mpe_protocol_type = mpe_endat;

	/* read encoder info at low frequency so that cable length won't affect */
	cmd_supplement.frequency = 200 * 1000;
	endat_process_host_command(100, &cmd_supplement, priv);

	if (endat_get_encoder_info(priv) < 0) {
		//CONSOLEUtilsPrintf("EnDat initialization failed\n");
		return;
	}

	/* default frequency - 8MHz for 2.2 encoders, 1MHz for 2.1 encoders */
	if (priv->cmd_set_2_2)
		cmd_supplement.frequency = 8 * 1000 * 1000;
	else
		cmd_supplement.frequency = 1 * 1000 * 1000;
	endat_process_host_command(100, &cmd_supplement, priv);

	endat_start_continuous_mode(priv);

	endat_recvd_process(priv, 1, &endat_format_data_mtrctrl);

	unsigned long long max = pow(2, priv->single_turn_res);
	union position position, previous_position;
	unsigned int pos_cnt=0;

	CONSOLEUtilsPrintf("\nEnDat 2.2\n");

	while(1)
	{
		endat_recvd_process(priv, 1, &endat_format_data_mtrctrl);
		if (priv->type == rotary)
			position.angle = ((float) endat_format_data_mtrctrl.position_addinfo.position.position) / (float)max * (float)360;
		else
			position.length = endat_format_data_mtrctrl.position_addinfo.position.position * priv->step;

		TaskOSAL_sleep(50);

		// simplified method to detect that the decoder is disconnected
		if(position.angle == previous_position.angle)
		{
			pos_cnt++;
			if(pos_cnt>10)
			{
				CONSOLEUtilsPrintf("\n");
				return;
			}
		}
		else
			pos_cnt=0;
		previous_position.angle = position.angle;

		//CONSOLEUtilsPrintf("angle=%f\n", position.angle);
		CONSOLEUtilsPrintf("angle=%f\r", position.angle);
	}

}

/**
 * @brief Endat wait for initalitzation function
 *
 *
 */
int mpe_endat_wait_initialization(struct endat_priv *priv, unsigned timeout)
{
	int i;
	struct endat_pruss_xchg *pruss_xchg = priv->pruss_xchg;

	for (i = 0; i < timeout; i++)
		if(pruss_xchg->config.status & 1)
			break;
		else
			//DelayOSAL_milliSeconds(1);
			TaskOSAL_sleep(2);


	if (i == timeout)
		return -1;
	return 0;
}


/**
 * @brief Endat clock configuration function
 *
 *
 */
static int endat_calc_clock(unsigned freq, struct endat_clk_cfg *clk_cfg)
{
	unsigned ns;

	if (freq > 16000000 || (freq > 12000000 && freq < 16000000)) {
		CONSOLEUtilsPrintf("| ERROR: frequency above 16MHz, between 12 & 16MHz not allowed\n|\n|\n");
		return -1;
	}

	if ((freq != 16000000) && (ENDAT_INPUT_CLOCK_FREQUENCY % (freq * 8)))
		CONSOLEUtilsPrintf("| WARNING: exact clock divider is not possible, frequencies set would be tx: %u\trx: %u\n",
					ENDAT_INPUT_CLOCK_FREQUENCY / (ENDAT_INPUT_CLOCK_FREQUENCY / freq),
					ENDAT_INPUT_CLOCK_FREQUENCY / (ENDAT_INPUT_CLOCK_FREQUENCY / (freq * 8)));

	ns = 2 * 1000000000 / freq; /* rx arm >= 2 clock */
	/* should be divisible by 5 */
	if (ns % 5)
		ns /= 5, ns += 1,  ns *= 5;

	clk_cfg->tx_div = ENDAT_INPUT_CLOCK_FREQUENCY / freq - 1;
	clk_cfg->rx_div = ENDAT_INPUT_CLOCK_FREQUENCY / (freq * 8) - 1;
	clk_cfg->rx_en_cnt = ns;
	clk_cfg->rx_div_attr = ENDAT_RX_SAMPLE_SIZE;
	if (freq == 16000000)
		clk_cfg->rx_div_attr |= ENDAT_RX_SESQUI_DIV;

	return 0;
}


/**
 * @brief EnDat host command helper function
 *
 *
 */
static void endat_process_host_command(int cmd, struct cmd_supplement *cmd_supplement, struct endat_priv *priv)
{
	struct endat_clk_cfg clk_cfg;
	unsigned short prop_delay = endat_get_prop_delay(priv);

	/* clock configuration */
	if (cmd == 100) {
		if (endat_calc_clock(cmd_supplement->frequency, &clk_cfg) < 0)
			return;

		endat_config_clock(priv, &clk_cfg);
		priv->rx_en_cnt = clk_cfg.rx_en_cnt;

		if (prop_delay > priv->rx_en_cnt) {
			unsigned short dis = (prop_delay - priv->rx_en_cnt) * 2 / priv->rx_en_cnt;

			endat_config_rx_arm_cnt(priv, prop_delay);
			/* propagation delay - 2T */
			endat_config_rx_clock_disable(priv, dis);
		} else
			endat_config_rx_clock_disable(priv, 0);

		/* set tST to 2us if frequency > 1MHz, else turn it off */
		if (cmd_supplement->frequency >= 1000000)
			cmd_supplement->frequency = 2000;
		else
			cmd_supplement->frequency = 0;
		endat_process_host_command(103, cmd_supplement, priv);
	} else if (cmd == 102) {
		priv->raw_data ^= 1;
	} else if (cmd == 103) {
		unsigned delay;

		delay = endat_do_sanity_tst_delay(cmd_supplement->frequency);
		if (delay <= (unsigned short)~0)
			endat_config_tst_delay(priv, (unsigned short) delay);
	} else
		CONSOLEUtilsPrintf("| ERROR: non host command being requested to be handled as host command\n|\n|\n");
}

/**
 * @brief EnDat delay helper configuration function
 *
 *
 */
static unsigned endat_do_sanity_tst_delay(unsigned delay)
{
	/* (unsigned short)~0 is also a multiple of 5 */
	if (delay > (unsigned short)~0) {
		CONSOLEUtilsPrintf("| ERROR: delay greater than %uns, enter lesser value\n|\n|\n", (unsigned short)~0);
		return delay;
	}

	if (delay % 5) {
		delay += 5, delay /= 5, delay *= 5;
		CONSOLEUtilsPrintf("| WARNING: delay not multiple of 5ns, rounding to %uns\n|\n|\n", delay);
	}

	return delay;
}
