/*
* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
*
*    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include "pruicss.h"
#include "hw_types.h"
#include "gpio.h"
#include "console_utils.h"
#include "osdrv_osal.h"
#include "osdrv_pruss.h"
#include "hw_icss.h"
#include "hw_icss_pru_ctrl.h"
#include "hw_icss_cfg.h"
#include "pruicss_v1.h"
#include "am437x.h"
#include "hw_control_am43xx.h"
#include "hw_cm_dpll.h"
#include "hw_cm_wkup.h"
#include "board_gpio.h"
#include "hw_icss_intc.h"
#include "hw_icss_iep.h"
#include "hw_edma_tpcc.h"
#include "hw_pwmss_ecap.h"
#include "timpe_pruss_intc_mapping.h"
#include "mpe_common.h"

#include "hiperface_dsl.h"
#include "Hiperface_DSL_header_bin.h"

#define ENDAT_EN						(1<<26)
#define SHIFT_REG_EN					(0<<26)
#define GPO_MODE						(1<<14)
#define GPI_MODE						(0x02)
#define GPO_DIV1						(0x04<<20) // 3
#define GPO_DIV0						(0x0e<<15) // 8
#define GPI_DIV1						(0x04<<8) // 3
#define GPI_DIV0						(0x0e<<3) // 8

#define ENDAT_TX_CFG					(0x10 | (23<<16)) // OCP as source clock, div 24
#define ENDAT_RX_CFG					(0x10 | (2<<16) | 7) // OCP as source clock, div 24, 8x oversampling
#define ENDAT_CHCFG0					(0<<16)
#define ENDAT_CHCFG1					(0<<16)

#define ENC_5B6B						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x00000100)
#define ENC_3B4B						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x00000160)
#define DEC_5B6B						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x00000120)
#define DEC_3B4B						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x00000168)
#define BITCNT							(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x00000178)
#define LUT_CRC5						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x478)
#define LUT_CRC16						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x278)
#define LUT_BITTOBYTE					(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x5a4)
#define LUT_RSSI						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x5E4)
#define LUT_EE							(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr+0x5E4+256)

//****************************************************
//Internal Register Interface Memory offsets
//****************************************************
#define MASTER_REGS_SIZE				(0x100)
#define MASTER_REGS						(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr)
#define TEST_PATTERN0					(MASTER_REGS+0xac)
#define PIMEM							(MASTER_REGS+0xd8)
#define ECAP_OFFSETS					(MASTER_REGS+0xce)
#define SYNC_CTRL						(MASTER_REGS+0x01)
#define VERSION							(MASTER_REGS+0x0B)
#define MAXDEV_H						(MASTER_REGS+0x3A)
#define MAXDEV_L						(MASTER_REGS+0x3B)
#define MAXDEV_THRESH_H					(MASTER_REGS+0x42)
#define PC_ADD_H						(MASTER_REGS+0x28)
#define PC_OFF_H						(MASTER_REGS+0x2A)
#define ACC_ERR_CNT_THRESH				(MASTER_REGS+0x41)

#define	EDMA_PARAM_0	0x4000
#define PSEUDO_PRUIMEM0					0x00002100
#define PRU0_IMEM						(0x00034000)
#define INVAL							0xFF

extern PRUICSS_Handle handle, pruIcss0Handle;
extern int mpe_protocol_type;

void HIPERFACE_ALL();
void SCU(void);
void ConfigureECAP(void);
void ConfigureIEP(void);
void ConfigureEDMA(void);
void GenerateMemoryImage(void);
void ENC_DEC_LUT(void);
void GenerateBitCntLUT(void);
void GenerateCRC5LUT(void);
void GenerateCRC16LUT(void);
void GenerateBitToByteLUT(void);
void GenerateEXTRAEDGELUT(void);
void ConfigureRegisterIF(void);
void GenerateRSSILUT(void);


/**
 * @brief Function called by MPEncoderTask() to establish communciation with Encoder
 *
 *
 */
void MPEncoderHiperfaceDSL(void)
{
	unsigned int temp32, pos_cnt;
	float position=0, previous_position=0;

	// stop PRU firmware
	PRUICSS_pruDisable(pruIcss0Handle, 0);
	// set program counter reset value to 0. Only required if the CCS debugger has re-asigned the reset value
	temp32 = HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0));
	temp32 &= ~(0xffff<<16);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0),  temp32);
	PRUICSS_pruReset(pruIcss0Handle, 0);

    // multiplexer adapter configuration
	GPIOPinWrite(SOC_GPIO2_REG, 12, GPIO_PIN_LOW);
    GPIOPinWrite(SOC_GPIO1_REG, 13, GPIO_PIN_HIGH);
    GPIOPinWrite(SOC_GPIO1_REG, 14, GPIO_PIN_HIGH);

    // clear pending events in ICSS0/1 INTC
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR0, HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR0));
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR1, HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR1));
    HW_WR_REG32(((PRUICSS_HwAttrs *)(handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR0, HW_RD_REG32(((PRUICSS_HwAttrs *)(handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR0));
    HW_WR_REG32(((PRUICSS_HwAttrs *)(handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR1, HW_RD_REG32(((PRUICSS_HwAttrs *)(handle->hwAttrs))->baseAddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR1));

	// configure external pinmux
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MMC0_DAT3), (CONTROL_CONF_MUXMODE(5)));	// gpio2_26 / ENDAT_TXEN

	// configure internal pinmux
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + HW_ICSS_CFG_GPCFG0,  0x1<<26);

    // Stop EnDAT TX: Tx_global_reinit R31.t19
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_DEBUG(0) + (31*4),  1<<19);
    // RX_disable R30
    HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_DEBUG(0) + (30*4),  0);

	// clear ICSS0 PRU0/1 data RAM
	memset((void *)(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x0000), 0, (4*1024));
	memset((void *)(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x2000), 0, (4*1024));

	HIPERFACE_ALL();

	// load firmware
	memcpy((void *)(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x34000), &Hiperface_DSL2_0[0] + 0x0000, 0x1000);
	//ICSS_0/PRU1 memory
	memcpy((void *)(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x2100), &Hiperface_DSL2_0[1023-10], 0x600);
	memcpy((void *)(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x2700), &Hiperface_DSL2_0[1385-10], 0x600);

	TaskOSAL_sleep(100);

	// start firmware: this is required if CCS debugger has reconfigured the initial settings
	temp32 = HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0));
	temp32 &= ~(1<<8); //clear single step bit
	temp32 |= (1<<1); // set PRU enable bit
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0),  temp32);

	TaskOSAL_sleep(100);

	// PRU goes to HALT if Hiperface DSL was not detected
	if(!(HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0)) & 0x2))
		return;

	mpe_protocol_type = mpe_hiperface_dsl;
	CONSOLEUtilsPrintf("\nHiperface DSL\n");

	while(1)
	{
		TaskOSAL_sleep(50);
		// read position
		temp32 = HW_RD_REG8(0x54440000 + 0x1B)<<16 | HW_RD_REG8(0x54440000 + 0x1c)<<8 | HW_RD_REG8(0x54440000 + 0x1d)<<0;
		position = (float) (temp32 & 0x3FFFF) / (float) 0x40000 * (float) 360;

		// Check if position is changing
		if(position == previous_position)
		{
			pos_cnt++;
			if(pos_cnt>10)
			{
				CONSOLEUtilsPrintf("\n");
				return;
			}
		}
		else
			pos_cnt=0;
		previous_position = position;

		CONSOLEUtilsPrintf("angle=%f\r", position);
	}
}

/**
 * @brief Configures the PRU-ICSS0 and encoder hardware to operated with
 * Hiperface DSL encoder
 *
 */
void HIPERFACE_ALL()
{
	conf_225MHZ();

	SCU();

	ConfigureECAP();
	ConfigureIEP();

	ConfigureEDMA();

	PRUICSS_IntcInitData pruss1_intc_initdata = PRUSS_INTC_INITDATA;
	PRUICSS_IntcInitData pruss0_intc_initdata = PRUSS_INTC_HIPERFACEDSL;
	PRUICSS_pruIntcInit(handle, &pruss1_intc_initdata);
	PRUICSS_pruIntcInit(pruIcss0Handle, &pruss0_intc_initdata);

	ConfigureConstantTable();
	GenerateMemoryImage();

	// enable cyclecount register
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0), HW_RD_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0)) | (1<<3));
}


/**
 * @brief serial capture unit configuration function
 *
 *
 */
void SCU(void)
{
	// configure SCU to run with 9.735 MHz -> DIV 24, enable ENDAT, use channel 0, set clockmode to free run
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + HW_ICSS_CFG_GPCFG0,  (ENDAT_EN | GPO_MODE | GPO_DIV1 | GPO_DIV0 | GPI_DIV1 | GPI_DIV0));

	// set source clk to OCP and a div of 24 and configure channel 0,1,2
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDTXCFG,  ENDAT_TX_CFG);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDRXCFG,  ENDAT_RX_CFG);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH0CFG0,  ENDAT_CHCFG0);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH0CFG1,  ENDAT_CHCFG1);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH1CFG0,  ENDAT_CHCFG0);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH1CFG1,  ENDAT_CHCFG1);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH2CFG0,  ENDAT_CHCFG0);
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + ICSS_P0EDCH2CFG1,  ENDAT_CHCFG1);

	/* configure PRU1 to use GPI */
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + HW_ICSS_CFG_GPCFG1,  0);
}

/**
 * @brief ECAP configuration function
 *
 *
 */
void ConfigureECAP(void)
{
	unsigned int temp32;
	// enable ecap
	HW_WR_REG32(SOC_CM_PER_REG + PRCM_CM_PER_EPWMSS1_CLKCTRL, 0x02);
	do {
		temp32 = HW_RD_REG32(SOC_CM_PER_REG + PRCM_CM_PER_EPWMSS1_CLKCTRL) & (3<<16);
	}
	while(temp32);
	/* Use eCAP1 and J24 for input */
	HW_WR_REG16(SOC_PWMSS1_ECAP_REG + PWMSS_ECAP_ECCTL1, ((1<<8) | (1<<1)));
	HW_WR_REG16(SOC_PWMSS1_ECAP_REG +  + PWMSS_ECAP_ECCTL2, ((1<<4)));
	HW_WR_REG16(SOC_PWMSS1_ECAP_REG +  + PWMSS_ECAP_ECEINT, (1<<1));

	HW_WR_REG32(SOC_CM_PER_REG + PRCM_CM_PER_EPWMSS2_CLKCTRL, 0x02);
	do {
		temp32 = HW_RD_REG32(SOC_CM_PER_REG + PRCM_CM_PER_EPWMSS2_CLKCTRL) & (3<<16);
	}
	while(temp32);
	/* Use eCAP2 and M24 for input */
	HW_WR_REG16(SOC_PWMSS2_ECAP_REG + PWMSS_ECAP_ECCTL1, ((1<<8) | (0<<1) | (1<<0)));
	HW_WR_REG16(SOC_PWMSS2_ECAP_REG + PWMSS_ECAP_ECCTL2, ((1<<4) | (1<<5)));
	HW_WR_REG16(SOC_PWMSS2_ECAP_REG + PWMSS_ECAP_ECEINT, (1<<1));
}

/**
 * @brief IEP configuration function
 *
 *
 */
void ConfigureIEP(void)
{
	// enable cnt and inc. by 1
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_IEP + HW_ICSS_IEP_GLOBAL_CFG, (1<<4) | (1<<0));
	// no compensation
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_IEP + HW_ICSS_IEP_COMPEN, 0);
	// use OCP as IEP CLK source
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + HW_ICSS_CFG_IEPCLK, 1);
}


/**
 * @brief EDMA configuration function
 *
 *
 */
void ConfigureEDMA(void)
{
	unsigned int temp32;
		// enable edma
	HW_WR_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPCC_CLKCTRL, 0x02);
	do {
		temp32 = HW_RD_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPCC_CLKCTRL) & (3<<16);
	}while(temp32);
	HW_WR_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPTC0_CLKCTRL, 0x02);
	do {
		temp32 = HW_RD_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPTC0_CLKCTRL) & (3<<16);
	}while(temp32);
	HW_WR_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPTC1_CLKCTRL, 0x02);
	do {
		temp32 = HW_RD_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPTC1_CLKCTRL) & (3<<16);
	}while(temp32);
	HW_WR_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPTC2_CLKCTRL, 0x02);
	do {
		temp32 = HW_RD_REG32(SOC_CM_PER_REG + PRCM_CM_PER_TPTC2_CLKCTRL) & (3<<16);
	}while(temp32);
	// DMACH39->PARAM0
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(39), (0<<5));
	// DMACH1->PARAM6
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(1), (6<<5));
	// DMACH0->PARAM6
	//HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(0), (6<<5));
	// DMACH2->PARAM1
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(2), (1<<5));
	// DMACH3->PARAM3
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(3), (3<<5));
	// DMACH62->PARAM2
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(62), (2<<5));
	// DMACH4->PARAM4
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(4), (4<<5));
	// DMACH5->PARAM5
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(5), (5<<5));
	// DMACH6->PARAM6
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(6), (6<<5));
	// DMACH7->PARAM7
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DCHMAP(7), (7<<5));
	// set ecap1 event to event queue 0
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_DMAQNUM(4), (0<<28));
	// configure PaRAM0 to reset PWMSS2 CNT
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x00, ((1<<2) | (1<<3) | (1<<22) | (2<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x04, (SOC_PWMSS1_ECAP_REG+0x00));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x08, (4 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x0c, /*(PRUSS_ICSS + ICSS_0 + 0x00 + 0x70 )*/(SOC_PWMSS2_ECAP_REG));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x14, (/*0x20*/ 0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x1c, 1);
	// configure PaRAM1 to load PWMSS1.ecap1 to pdmem00
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x00, ((1<<2) | (1<<3) | (0<<22) | (3<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x04, (SOC_PWMSS1_ECAP_REG+0x08));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x08, (4 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x0c, (((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x00 + 0xbc));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x14, (0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20+0x1c, 1);
	// configure PaRAM2 to load PWMSS1.cnt to pdmem00
	HW_WR_REG32(SOC_PWMSS1_ECAP_REG+0x04, 0xffffffff);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x00, ((1<<2) | (1<<3) | (1<<22) | (3<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x04, (SOC_PWMSS1_ECAP_REG/*+0x08*/));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x08, (4 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x0c, (((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x00 + 0xc0));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x14, (0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*2+0x1c, 1);
	// configure PaRAM3 to signalize PRU core //reset PWMSS2 INT
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x00, ((1<<2) | (1<<3) | (0<<22) | (2<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x04, (SOC_PWMSS1_ECAP_REG+0x04));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x08, ((1 | (1<<16))));//(2 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x0c, ((((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + 0x00 + 0xaa)));//(PWMSS2_ECAP+0x30));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x14, (0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*3+0x1c, 1);
	// configure PaRAM4 to load memory offset for IMEM transfer (destination)
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x00, ((1<<2) | (1<<3) | (1<<22) | (7<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x04, (PIMEM+4));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x08, (4 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x0c, (SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x0c));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x14, (0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*4+0x1c, 1);
	// configure PaRAM5 to RESUME PRU00 code
	HW_WR_REG32(SOC_PWMSS2_ECAP_REG+0x04, 0x0000000B); // This value is written into the PRU CTRL register to restart PRU. It is located in PWMSS!
	//HW_WR_REG32(SOC_PWMSS2_ECAP_REG+0x04, 0x03F3000B); // This value is written into the PRU CTRL register to restart PRU. It is located in PWMSS!
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x00, ((1<<2) | (1<<3) | (0<<22) | (2<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x04, (SOC_PWMSS2_ECAP_REG+0x04));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x08, (4 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x0c, (((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x14, (0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*5+0x1c, 1);
	// configure PaRAM6 to load memory offsets and size for IMEM transfer
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x00, ((1<<2) | (1<<3) | (1<<22) | (7<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x04, (PIMEM));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x08, (12 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x0c, (SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x04));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x14, (0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*6+0x1c, 1);
	// configure PaRAM7 to load memory from PDMEM00 to IMEM00
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x00, ((1<<2) | (1<<3) | (1<<22) | (5<<12)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x04, (((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PSEUDO_PRUIMEM0));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x08, (0x360 | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x0c, (((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU0_IMEM));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x10, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x14, (0xffff | (1<<16)));
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x18, 0);
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_PARAM_0+0x20*7+0x1c, 1);
	// set source event for DMA trigger
	temp32 = 0;//RD_MEM_32(EDMA_EESRH);
	//HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_EESRH, (temp32 | (1<<(39-32)) | (1<<(62-32)))); //eCAPEVT1 & eCAPEVT2
	HW_WR_REG32(SOC_EDMA3CC_REG + EDMA_TPCC_EESR, (1<<1));	//pr1_host6
}

/**
 * @brief memory configuration function
 *
 *
 */
void GenerateMemoryImage(void)
{
	ENC_DEC_LUT();
	GenerateBitCntLUT();
	GenerateCRC5LUT();
	GenerateCRC16LUT();
	GenerateBitToByteLUT();
	GenerateRSSILUT();
	GenerateEXTRAEDGELUT();
	ConfigureRegisterIF();
}


/**
 * @brief look up table configuration function
 *
 *
 */
void ENC_DEC_LUT(void)
{
	// Encoding LUTs (input MSB first, output LSB first)
	// LUT 5b/6b encoding
	HW_WR_REG8(ENC_5B6B+0x00, 0x18);
	HW_WR_REG8(ENC_5B6B+0x01, 0x22);
	HW_WR_REG8(ENC_5B6B+0x02, 0x12);
	HW_WR_REG8(ENC_5B6B+0x03, 0x31);
	HW_WR_REG8(ENC_5B6B+0x04, 0x0a);
	HW_WR_REG8(ENC_5B6B+0x05, 0x29);
	HW_WR_REG8(ENC_5B6B+0x06, 0x19);
	HW_WR_REG8(ENC_5B6B+0x07, 0x07);
	HW_WR_REG8(ENC_5B6B+0x08, 0x06);
	HW_WR_REG8(ENC_5B6B+0x09, 0x25);
	HW_WR_REG8(ENC_5B6B+0x0a, 0x15);
	HW_WR_REG8(ENC_5B6B+0x0b, 0x34);
	HW_WR_REG8(ENC_5B6B+0x0c, 0x0d);
	HW_WR_REG8(ENC_5B6B+0x0d, 0x2c);
	HW_WR_REG8(ENC_5B6B+0x0e, 0x1c);
	HW_WR_REG8(ENC_5B6B+0x0f, 0x28);
	HW_WR_REG8(ENC_5B6B+0x10, 0x24);
	HW_WR_REG8(ENC_5B6B+0x11, 0x23);
	HW_WR_REG8(ENC_5B6B+0x12, 0x13);
	HW_WR_REG8(ENC_5B6B+0x13, 0x32);
	HW_WR_REG8(ENC_5B6B+0x14, 0x0b);
	HW_WR_REG8(ENC_5B6B+0x15, 0x2a);
	HW_WR_REG8(ENC_5B6B+0x16, 0x1a);
	HW_WR_REG8(ENC_5B6B+0x17, 0x05);
	HW_WR_REG8(ENC_5B6B+0x18, 0x0c);
	HW_WR_REG8(ENC_5B6B+0x19, 0x26);
	HW_WR_REG8(ENC_5B6B+0x1a, 0x16);
	HW_WR_REG8(ENC_5B6B+0x1b, 0x09);
	HW_WR_REG8(ENC_5B6B+0x1c, 0x30);
	HW_WR_REG8(ENC_5B6B+0x1d, 0x11);
	HW_WR_REG8(ENC_5B6B+0x1e, 0x21);
	HW_WR_REG8(ENC_5B6B+0x1f, 0x14);

	// LUT 3b/64 encoding
	HW_WR_REG8(ENC_3B4B+0x00, 0x04);
	HW_WR_REG8(ENC_3B4B+0x01, 0x09);
	HW_WR_REG8(ENC_3B4B+0x02, 0x05);
	HW_WR_REG8(ENC_3B4B+0x03, 0x03);
	HW_WR_REG8(ENC_3B4B+0x04, 0x02);
	HW_WR_REG8(ENC_3B4B+0x05, 0x0a);
	HW_WR_REG8(ENC_3B4B+0x06, 0x06);
	HW_WR_REG8(ENC_3B4B+0x07, 0x01);

	// input is with LSB first, output is MSB first
	HW_WR_REG8(DEC_5B6B+0x00, INVAL);
	HW_WR_REG8(DEC_5B6B+0x01, INVAL);
	HW_WR_REG8(DEC_5B6B+0x02, INVAL);
	HW_WR_REG8(DEC_5B6B+0x03, INVAL);
	HW_WR_REG8(DEC_5B6B+0x04, INVAL);
	HW_WR_REG8(DEC_5B6B+0x05, 0x17);
	HW_WR_REG8(DEC_5B6B+0x06, 0x08);
	HW_WR_REG8(DEC_5B6B+0x07, 0x07);
	HW_WR_REG8(DEC_5B6B+0x08, INVAL);
	HW_WR_REG8(DEC_5B6B+0x09, 0x1b);
	HW_WR_REG8(DEC_5B6B+0x0a, 0x04);
	HW_WR_REG8(DEC_5B6B+0x0b, 0x14);
	HW_WR_REG8(DEC_5B6B+0x0c, 0x18);
	HW_WR_REG8(DEC_5B6B+0x0d, 0x0c);
	HW_WR_REG8(DEC_5B6B+0x0e, 0x1c);
	HW_WR_REG8(DEC_5B6B+0x0f, 0x1c);
	HW_WR_REG8(DEC_5B6B+0x10, INVAL);
	HW_WR_REG8(DEC_5B6B+0x11, 0x1d);
	HW_WR_REG8(DEC_5B6B+0x12, 0x02);
	HW_WR_REG8(DEC_5B6B+0x13, 0x12);
	HW_WR_REG8(DEC_5B6B+0x14, 0x1f);
	HW_WR_REG8(DEC_5B6B+0x15, 0x0a);
	HW_WR_REG8(DEC_5B6B+0x16, 0x1a);
	HW_WR_REG8(DEC_5B6B+0x17, 0x0f);
	HW_WR_REG8(DEC_5B6B+0x18, 0x00);
	HW_WR_REG8(DEC_5B6B+0x19, 0x06);
	HW_WR_REG8(DEC_5B6B+0x1a, 0x16);
	HW_WR_REG8(DEC_5B6B+0x1b, 0x10);
	HW_WR_REG8(DEC_5B6B+0x1c, 0x0e);
	HW_WR_REG8(DEC_5B6B+0x1d, 0x01);
	HW_WR_REG8(DEC_5B6B+0x1e, 0x1e);
	HW_WR_REG8(DEC_5B6B+0x1f, INVAL);
	HW_WR_REG8(DEC_5B6B+0x20, INVAL);
	HW_WR_REG8(DEC_5B6B+0x21, 0x1e);
	HW_WR_REG8(DEC_5B6B+0x22, 0x01);
	HW_WR_REG8(DEC_5B6B+0x23, 0x11);
	HW_WR_REG8(DEC_5B6B+0x24, 0x10);
	HW_WR_REG8(DEC_5B6B+0x25, 0x09);
	HW_WR_REG8(DEC_5B6B+0x26, 0x19);
	HW_WR_REG8(DEC_5B6B+0x27, 0x00);
	HW_WR_REG8(DEC_5B6B+0x28, 0x0f);
	HW_WR_REG8(DEC_5B6B+0x29, 0x05);
	HW_WR_REG8(DEC_5B6B+0x2a, 0x15);
	HW_WR_REG8(DEC_5B6B+0x2b, 0x1f);
	HW_WR_REG8(DEC_5B6B+0x2c, 0x0d);
	HW_WR_REG8(DEC_5B6B+0x2d, 0x02);
	HW_WR_REG8(DEC_5B6B+0x2e, 0x1d);
	HW_WR_REG8(DEC_5B6B+0x2f, INVAL);
	HW_WR_REG8(DEC_5B6B+0x30, 0x1c);
	HW_WR_REG8(DEC_5B6B+0x31, 0x03);
	HW_WR_REG8(DEC_5B6B+0x32, 0x13);
	HW_WR_REG8(DEC_5B6B+0x33, 0x18);
	HW_WR_REG8(DEC_5B6B+0x34, 0x0b);
	HW_WR_REG8(DEC_5B6B+0x35, 0x04);
	HW_WR_REG8(DEC_5B6B+0x36, 0x1b);
	HW_WR_REG8(DEC_5B6B+0x37, INVAL);
	HW_WR_REG8(DEC_5B6B+0x38, 0x07);
	HW_WR_REG8(DEC_5B6B+0x39, 0x08);
	HW_WR_REG8(DEC_5B6B+0x3a, 0x17);
	HW_WR_REG8(DEC_5B6B+0x3b, INVAL);
	HW_WR_REG8(DEC_5B6B+0x3c, INVAL);
	HW_WR_REG8(DEC_5B6B+0x3d, INVAL);
	HW_WR_REG8(DEC_5B6B+0x3e, INVAL);
	HW_WR_REG8(DEC_5B6B+0x3f, INVAL);

	// LUT 3b/4b decoding
	// input is with LSB first, output is MSB first
	HW_WR_REG8(DEC_3B4B+0x00, INVAL);
	HW_WR_REG8(DEC_3B4B+0x01, (0x7 << 5));
	HW_WR_REG8(DEC_3B4B+0x02, (0x4 << 5));
	HW_WR_REG8(DEC_3B4B+0x03, (0x3 << 5));
	HW_WR_REG8(DEC_3B4B+0x04, (0x0 << 5));
	HW_WR_REG8(DEC_3B4B+0x05, (0x2 << 5));
	HW_WR_REG8(DEC_3B4B+0x06, (0x6 << 5));
	HW_WR_REG8(DEC_3B4B+0x07, (0x7 << 5));
	HW_WR_REG8(DEC_3B4B+0x08, (0x7 << 5));
	HW_WR_REG8(DEC_3B4B+0x09, (0x1 << 5));
	HW_WR_REG8(DEC_3B4B+0x0a, (0x5 << 5));
	HW_WR_REG8(DEC_3B4B+0x0b, (0x0 << 5));
	HW_WR_REG8(DEC_3B4B+0x0c, (0x3 << 5));
	HW_WR_REG8(DEC_3B4B+0x0d, (0x4 << 5));
	HW_WR_REG8(DEC_3B4B+0x0e, (0x7 << 5));
	HW_WR_REG8(DEC_3B4B+0x0f, INVAL);
}


/**
 * @brief look up table configuration function
 *
 *
 */
void GenerateBitCntLUT(void)
{
	HW_WR_REG8(BITCNT+0x0, 0);
	HW_WR_REG8(BITCNT+0x1, 1);
	HW_WR_REG8(BITCNT+0x2, 1);
	HW_WR_REG8(BITCNT+0x3, 2);
	HW_WR_REG8(BITCNT+0x4, 1);
	HW_WR_REG8(BITCNT+0x5, 2);
	HW_WR_REG8(BITCNT+0x6, 2);
	HW_WR_REG8(BITCNT+0x7, 3);
	HW_WR_REG8(BITCNT+0x8, 1);
	HW_WR_REG8(BITCNT+0x9, 2);
	HW_WR_REG8(BITCNT+0xa, 2);
	HW_WR_REG8(BITCNT+0xb, 3);
	HW_WR_REG8(BITCNT+0xc, 2);
	HW_WR_REG8(BITCNT+0xd, 3);
	HW_WR_REG8(BITCNT+0xe, 3);
	HW_WR_REG8(BITCNT+0xf, 4);
	HW_WR_REG8(BITCNT+0x10, 1);
	HW_WR_REG8(BITCNT+0x11, 2);
	HW_WR_REG8(BITCNT+0x12, 2);
	HW_WR_REG8(BITCNT+0x13, 3);
	HW_WR_REG8(BITCNT+0x14, 2);
	HW_WR_REG8(BITCNT+0x15, 3);
	HW_WR_REG8(BITCNT+0x16, 3);
	HW_WR_REG8(BITCNT+0x17, 4);
	HW_WR_REG8(BITCNT+0x18, 2);
	HW_WR_REG8(BITCNT+0x19, 3);
	HW_WR_REG8(BITCNT+0x1a, 3);
	HW_WR_REG8(BITCNT+0x1b, 4);
	HW_WR_REG8(BITCNT+0x1c, 3);
	HW_WR_REG8(BITCNT+0x1d, 4);
	HW_WR_REG8(BITCNT+0x1e, 4);
	HW_WR_REG8(BITCNT+0x1f, 5);
	HW_WR_REG8(BITCNT+0x20, 1);
	HW_WR_REG8(BITCNT+0x21, 2);
	HW_WR_REG8(BITCNT+0x22, 2);
	HW_WR_REG8(BITCNT+0x23, 3);
	HW_WR_REG8(BITCNT+0x24, 2);
	HW_WR_REG8(BITCNT+0x25, 3);
	HW_WR_REG8(BITCNT+0x26, 3);
	HW_WR_REG8(BITCNT+0x27, 4);
	HW_WR_REG8(BITCNT+0x28, 2);
	HW_WR_REG8(BITCNT+0x29, 3);
	HW_WR_REG8(BITCNT+0x2a, 3);
	HW_WR_REG8(BITCNT+0x2b, 4);
	HW_WR_REG8(BITCNT+0x2c, 3);
	HW_WR_REG8(BITCNT+0x2d, 4);
	HW_WR_REG8(BITCNT+0x2e, 4);
	HW_WR_REG8(BITCNT+0x2f, 5);
	HW_WR_REG8(BITCNT+0x30, 2);
	HW_WR_REG8(BITCNT+0x31, 3);
	HW_WR_REG8(BITCNT+0x32, 3);
	HW_WR_REG8(BITCNT+0x33, 4);
	HW_WR_REG8(BITCNT+0x34, 3);
	HW_WR_REG8(BITCNT+0x35, 4);
	HW_WR_REG8(BITCNT+0x36, 4);
	HW_WR_REG8(BITCNT+0x37, 5);
	HW_WR_REG8(BITCNT+0x38, 3);
	HW_WR_REG8(BITCNT+0x39, 4);
	HW_WR_REG8(BITCNT+0x3a, 4);
	HW_WR_REG8(BITCNT+0x3b, 5);
	HW_WR_REG8(BITCNT+0x3c, 4);
	HW_WR_REG8(BITCNT+0x3d, 5);
	HW_WR_REG8(BITCNT+0x3e, 5);
	HW_WR_REG8(BITCNT+0x3f, 6);
	HW_WR_REG8(BITCNT+0x40, 1);
	HW_WR_REG8(BITCNT+0x41, 2);
	HW_WR_REG8(BITCNT+0x42, 2);
	HW_WR_REG8(BITCNT+0x43, 3);
	HW_WR_REG8(BITCNT+0x44, 2);
	HW_WR_REG8(BITCNT+0x45, 3);
	HW_WR_REG8(BITCNT+0x46, 3);
	HW_WR_REG8(BITCNT+0x47, 4);
	HW_WR_REG8(BITCNT+0x48, 2);
	HW_WR_REG8(BITCNT+0x49, 3);
	HW_WR_REG8(BITCNT+0x4a, 3);
	HW_WR_REG8(BITCNT+0x4b, 4);
	HW_WR_REG8(BITCNT+0x4c, 3);
	HW_WR_REG8(BITCNT+0x4d, 4);
	HW_WR_REG8(BITCNT+0x4e, 4);
	HW_WR_REG8(BITCNT+0x4f, 5);
	HW_WR_REG8(BITCNT+0x50, 2);
	HW_WR_REG8(BITCNT+0x51, 3);
	HW_WR_REG8(BITCNT+0x52, 3);
	HW_WR_REG8(BITCNT+0x53, 4);
	HW_WR_REG8(BITCNT+0x54, 3);
	HW_WR_REG8(BITCNT+0x55, 4);
	HW_WR_REG8(BITCNT+0x56, 4);
	HW_WR_REG8(BITCNT+0x57, 5);
	HW_WR_REG8(BITCNT+0x58, 3);
	HW_WR_REG8(BITCNT+0x59, 4);
	HW_WR_REG8(BITCNT+0x5a, 4);
	HW_WR_REG8(BITCNT+0x5b, 5);
	HW_WR_REG8(BITCNT+0x5c, 4);
	HW_WR_REG8(BITCNT+0x5d, 5);
	HW_WR_REG8(BITCNT+0x5e, 5);
	HW_WR_REG8(BITCNT+0x5f, 6);
	HW_WR_REG8(BITCNT+0x60, 2);
	HW_WR_REG8(BITCNT+0x61, 3);
	HW_WR_REG8(BITCNT+0x62, 3);
	HW_WR_REG8(BITCNT+0x63, 4);
	HW_WR_REG8(BITCNT+0x64, 3);
	HW_WR_REG8(BITCNT+0x65, 4);
	HW_WR_REG8(BITCNT+0x66, 4);
	HW_WR_REG8(BITCNT+0x67, 5);
	HW_WR_REG8(BITCNT+0x68, 3);
	HW_WR_REG8(BITCNT+0x69, 4);
	HW_WR_REG8(BITCNT+0x6a, 4);
	HW_WR_REG8(BITCNT+0x6b, 5);
	HW_WR_REG8(BITCNT+0x6c, 4);
	HW_WR_REG8(BITCNT+0x6d, 5);
	HW_WR_REG8(BITCNT+0x6e, 5);
	HW_WR_REG8(BITCNT+0x6f, 6);
	HW_WR_REG8(BITCNT+0x70, 3);
	HW_WR_REG8(BITCNT+0x71, 4);
	HW_WR_REG8(BITCNT+0x72, 4);
	HW_WR_REG8(BITCNT+0x73, 5);
	HW_WR_REG8(BITCNT+0x74, 4);
	HW_WR_REG8(BITCNT+0x75, 5);
	HW_WR_REG8(BITCNT+0x76, 5);
	HW_WR_REG8(BITCNT+0x77, 6);
	HW_WR_REG8(BITCNT+0x78, 4);
	HW_WR_REG8(BITCNT+0x79, 5);
	HW_WR_REG8(BITCNT+0x7a, 5);
	HW_WR_REG8(BITCNT+0x7b, 6);
	HW_WR_REG8(BITCNT+0x7c, 5);
	HW_WR_REG8(BITCNT+0x7d, 6);
	HW_WR_REG8(BITCNT+0x7e, 6);
	HW_WR_REG8(BITCNT+0x7f, 7);
	HW_WR_REG8(BITCNT+0x80, 1);
	HW_WR_REG8(BITCNT+0x81, 2);
	HW_WR_REG8(BITCNT+0x82, 2);
	HW_WR_REG8(BITCNT+0x83, 3);
	HW_WR_REG8(BITCNT+0x84, 2);
	HW_WR_REG8(BITCNT+0x85, 3);
	HW_WR_REG8(BITCNT+0x86, 3);
	HW_WR_REG8(BITCNT+0x87, 4);
	HW_WR_REG8(BITCNT+0x88, 2);
	HW_WR_REG8(BITCNT+0x89, 3);
	HW_WR_REG8(BITCNT+0x8a, 3);
	HW_WR_REG8(BITCNT+0x8b, 4);
	HW_WR_REG8(BITCNT+0x8c, 3);
	HW_WR_REG8(BITCNT+0x8d, 4);
	HW_WR_REG8(BITCNT+0x8e, 4);
	HW_WR_REG8(BITCNT+0x8f, 5);
	HW_WR_REG8(BITCNT+0x90, 2);
	HW_WR_REG8(BITCNT+0x91, 3);
	HW_WR_REG8(BITCNT+0x92, 3);
	HW_WR_REG8(BITCNT+0x93, 4);
	HW_WR_REG8(BITCNT+0x94, 3);
	HW_WR_REG8(BITCNT+0x95, 4);
	HW_WR_REG8(BITCNT+0x96, 4);
	HW_WR_REG8(BITCNT+0x97, 5);
	HW_WR_REG8(BITCNT+0x98, 3);
	HW_WR_REG8(BITCNT+0x99, 4);
	HW_WR_REG8(BITCNT+0x9a, 4);
	HW_WR_REG8(BITCNT+0x9b, 5);
	HW_WR_REG8(BITCNT+0x9c, 4);
	HW_WR_REG8(BITCNT+0x9d, 5);
	HW_WR_REG8(BITCNT+0x9e, 5);
	HW_WR_REG8(BITCNT+0x9f, 6);
	HW_WR_REG8(BITCNT+0xa0, 2);
	HW_WR_REG8(BITCNT+0xa1, 3);
	HW_WR_REG8(BITCNT+0xa2, 3);
	HW_WR_REG8(BITCNT+0xa3, 4);
	HW_WR_REG8(BITCNT+0xa4, 3);
	HW_WR_REG8(BITCNT+0xa5, 4);
	HW_WR_REG8(BITCNT+0xa6, 4);
	HW_WR_REG8(BITCNT+0xa7, 5);
	HW_WR_REG8(BITCNT+0xa8, 3);
	HW_WR_REG8(BITCNT+0xa9, 4);
	HW_WR_REG8(BITCNT+0xaa, 4);
	HW_WR_REG8(BITCNT+0xab, 5);
	HW_WR_REG8(BITCNT+0xac, 4);
	HW_WR_REG8(BITCNT+0xad, 5);
	HW_WR_REG8(BITCNT+0xae, 5);
	HW_WR_REG8(BITCNT+0xaf, 6);
	HW_WR_REG8(BITCNT+0xb0, 3);
	HW_WR_REG8(BITCNT+0xb1, 4);
	HW_WR_REG8(BITCNT+0xb2, 4);
	HW_WR_REG8(BITCNT+0xb3, 5);
	HW_WR_REG8(BITCNT+0xb4, 4);
	HW_WR_REG8(BITCNT+0xb5, 5);
	HW_WR_REG8(BITCNT+0xb6, 5);
	HW_WR_REG8(BITCNT+0xb7, 6);
	HW_WR_REG8(BITCNT+0xb8, 4);
	HW_WR_REG8(BITCNT+0xb9, 5);
	HW_WR_REG8(BITCNT+0xba, 5);
	HW_WR_REG8(BITCNT+0xbb, 6);
	HW_WR_REG8(BITCNT+0xbc, 5);
	HW_WR_REG8(BITCNT+0xbd, 6);
	HW_WR_REG8(BITCNT+0xbe, 6);
	HW_WR_REG8(BITCNT+0xbf, 7);
	HW_WR_REG8(BITCNT+0xc0, 2);
	HW_WR_REG8(BITCNT+0xc1, 3);
	HW_WR_REG8(BITCNT+0xc2, 3);
	HW_WR_REG8(BITCNT+0xc3, 4);
	HW_WR_REG8(BITCNT+0xc4, 3);
	HW_WR_REG8(BITCNT+0xc5, 4);
	HW_WR_REG8(BITCNT+0xc6, 4);
	HW_WR_REG8(BITCNT+0xc7, 5);
	HW_WR_REG8(BITCNT+0xc8, 3);
	HW_WR_REG8(BITCNT+0xc9, 4);
	HW_WR_REG8(BITCNT+0xca, 4);
	HW_WR_REG8(BITCNT+0xcb, 5);
	HW_WR_REG8(BITCNT+0xcc, 4);
	HW_WR_REG8(BITCNT+0xcd, 5);
	HW_WR_REG8(BITCNT+0xce, 5);
	HW_WR_REG8(BITCNT+0xcf, 6);
	HW_WR_REG8(BITCNT+0xd0, 3);
	HW_WR_REG8(BITCNT+0xd1, 4);
	HW_WR_REG8(BITCNT+0xd2, 4);
	HW_WR_REG8(BITCNT+0xd3, 5);
	HW_WR_REG8(BITCNT+0xd4, 4);
	HW_WR_REG8(BITCNT+0xd5, 5);
	HW_WR_REG8(BITCNT+0xd6, 5);
	HW_WR_REG8(BITCNT+0xd7, 6);
	HW_WR_REG8(BITCNT+0xd8, 4);
	HW_WR_REG8(BITCNT+0xd9, 5);
	HW_WR_REG8(BITCNT+0xda, 5);
	HW_WR_REG8(BITCNT+0xdb, 6);
	HW_WR_REG8(BITCNT+0xdc, 5);
	HW_WR_REG8(BITCNT+0xdd, 6);
	HW_WR_REG8(BITCNT+0xde, 6);
	HW_WR_REG8(BITCNT+0xdf, 7);
	HW_WR_REG8(BITCNT+0xe0, 3);
	HW_WR_REG8(BITCNT+0xe1, 4);
	HW_WR_REG8(BITCNT+0xe2, 4);
	HW_WR_REG8(BITCNT+0xe3, 5);
	HW_WR_REG8(BITCNT+0xe4, 4);
	HW_WR_REG8(BITCNT+0xe5, 5);
	HW_WR_REG8(BITCNT+0xe6, 5);
	HW_WR_REG8(BITCNT+0xe7, 6);
	HW_WR_REG8(BITCNT+0xe8, 4);
	HW_WR_REG8(BITCNT+0xe9, 5);
	HW_WR_REG8(BITCNT+0xea, 5);
	HW_WR_REG8(BITCNT+0xeb, 6);
	HW_WR_REG8(BITCNT+0xec, 5);
	HW_WR_REG8(BITCNT+0xed, 6);
	HW_WR_REG8(BITCNT+0xee, 6);
	HW_WR_REG8(BITCNT+0xef, 7);
	HW_WR_REG8(BITCNT+0xf0, 4);
	HW_WR_REG8(BITCNT+0xf1, 5);
	HW_WR_REG8(BITCNT+0xf2, 5);
	HW_WR_REG8(BITCNT+0xf3, 6);
	HW_WR_REG8(BITCNT+0xf4, 5);
	HW_WR_REG8(BITCNT+0xf5, 6);
	HW_WR_REG8(BITCNT+0xf6, 6);
	HW_WR_REG8(BITCNT+0xf7, 7);
	HW_WR_REG8(BITCNT+0xf8, 5);
	HW_WR_REG8(BITCNT+0xf9, 6);
	HW_WR_REG8(BITCNT+0xfa, 6);
	HW_WR_REG8(BITCNT+0xfb, 7);
	HW_WR_REG8(BITCNT+0xfc, 6);
	HW_WR_REG8(BITCNT+0xfd, 7);
	HW_WR_REG8(BITCNT+0xfe, 7);
	HW_WR_REG8(BITCNT+0xff, 8);
}

/**
 * @brief CRC configuration function
 *
 *
 */
void GenerateCRC5LUT(void)
{
	HW_WR_REG8(LUT_CRC5+0x0, 0x0);
	HW_WR_REG8(LUT_CRC5+0x1, 0x5);
	HW_WR_REG8(LUT_CRC5+0x2, 0xa);
	HW_WR_REG8(LUT_CRC5+0x3, 0xf);
	HW_WR_REG8(LUT_CRC5+0x4, 0x14);
	HW_WR_REG8(LUT_CRC5+0x5, 0x11);
	HW_WR_REG8(LUT_CRC5+0x6, 0x1e);
	HW_WR_REG8(LUT_CRC5+0x7, 0x1b);
	HW_WR_REG8(LUT_CRC5+0x8, 0xd);
	HW_WR_REG8(LUT_CRC5+0x9, 0x8);
	HW_WR_REG8(LUT_CRC5+0xa, 0x7);
	HW_WR_REG8(LUT_CRC5+0xb, 0x2);
	HW_WR_REG8(LUT_CRC5+0xc, 0x19);
	HW_WR_REG8(LUT_CRC5+0xd, 0x1c);
	HW_WR_REG8(LUT_CRC5+0xe, 0x13);
	HW_WR_REG8(LUT_CRC5+0xf, 0x16);
	HW_WR_REG8(LUT_CRC5+0x10, 0x1a);
	HW_WR_REG8(LUT_CRC5+0x11, 0x1f);
	HW_WR_REG8(LUT_CRC5+0x12, 0x10);
	HW_WR_REG8(LUT_CRC5+0x13, 0x15);
	HW_WR_REG8(LUT_CRC5+0x14, 0xe);
	HW_WR_REG8(LUT_CRC5+0x15, 0xb);
	HW_WR_REG8(LUT_CRC5+0x16, 0x4);
	HW_WR_REG8(LUT_CRC5+0x17, 0x1);
	HW_WR_REG8(LUT_CRC5+0x18, 0x17);
	HW_WR_REG8(LUT_CRC5+0x19, 0x12);
	HW_WR_REG8(LUT_CRC5+0x1a, 0x1d);
	HW_WR_REG8(LUT_CRC5+0x1b, 0x18);
	HW_WR_REG8(LUT_CRC5+0x1c, 0x3);
	HW_WR_REG8(LUT_CRC5+0x1d, 0x6);
	HW_WR_REG8(LUT_CRC5+0x1e, 0x9);
	HW_WR_REG8(LUT_CRC5+0x1f, 0xc);
	HW_WR_REG8(LUT_CRC5+0x20, 0x11);
	HW_WR_REG8(LUT_CRC5+0x21, 0x14);
	HW_WR_REG8(LUT_CRC5+0x22, 0x1b);
	HW_WR_REG8(LUT_CRC5+0x23, 0x1e);
	HW_WR_REG8(LUT_CRC5+0x24, 0x5);
	HW_WR_REG8(LUT_CRC5+0x25, 0x0);
	HW_WR_REG8(LUT_CRC5+0x26, 0xf);
	HW_WR_REG8(LUT_CRC5+0x27, 0xa);
	HW_WR_REG8(LUT_CRC5+0x28, 0x1c);
	HW_WR_REG8(LUT_CRC5+0x29, 0x19);
	HW_WR_REG8(LUT_CRC5+0x2a, 0x16);
	HW_WR_REG8(LUT_CRC5+0x2b, 0x13);
	HW_WR_REG8(LUT_CRC5+0x2c, 0x8);
	HW_WR_REG8(LUT_CRC5+0x2d, 0xd);
	HW_WR_REG8(LUT_CRC5+0x2e, 0x2);
	HW_WR_REG8(LUT_CRC5+0x2f, 0x7);
	HW_WR_REG8(LUT_CRC5+0x30, 0xb);
	HW_WR_REG8(LUT_CRC5+0x31, 0xe);
	HW_WR_REG8(LUT_CRC5+0x32, 0x1);
	HW_WR_REG8(LUT_CRC5+0x33, 0x4);
	HW_WR_REG8(LUT_CRC5+0x34, 0x1f);
	HW_WR_REG8(LUT_CRC5+0x35, 0x1a);
	HW_WR_REG8(LUT_CRC5+0x36, 0x15);
	HW_WR_REG8(LUT_CRC5+0x37, 0x10);
	HW_WR_REG8(LUT_CRC5+0x38, 0x6);
	HW_WR_REG8(LUT_CRC5+0x39, 0x3);
	HW_WR_REG8(LUT_CRC5+0x3a, 0xc);
	HW_WR_REG8(LUT_CRC5+0x3b, 0x9);
	HW_WR_REG8(LUT_CRC5+0x3c, 0x12);
	HW_WR_REG8(LUT_CRC5+0x3d, 0x17);
	HW_WR_REG8(LUT_CRC5+0x3e, 0x18);
	HW_WR_REG8(LUT_CRC5+0x3f, 0x1d);
	HW_WR_REG8(LUT_CRC5+0x40, 0x7);
	HW_WR_REG8(LUT_CRC5+0x41, 0x2);
	HW_WR_REG8(LUT_CRC5+0x42, 0xd);
	HW_WR_REG8(LUT_CRC5+0x43, 0x8);
	HW_WR_REG8(LUT_CRC5+0x44, 0x13);
	HW_WR_REG8(LUT_CRC5+0x45, 0x16);
	HW_WR_REG8(LUT_CRC5+0x46, 0x19);
	HW_WR_REG8(LUT_CRC5+0x47, 0x1c);
	HW_WR_REG8(LUT_CRC5+0x48, 0xa);
	HW_WR_REG8(LUT_CRC5+0x49, 0xf);
	HW_WR_REG8(LUT_CRC5+0x4a, 0x0);
	HW_WR_REG8(LUT_CRC5+0x4b, 0x5);
	HW_WR_REG8(LUT_CRC5+0x4c, 0x1e);
	HW_WR_REG8(LUT_CRC5+0x4d, 0x1b);
	HW_WR_REG8(LUT_CRC5+0x4e, 0x14);
	HW_WR_REG8(LUT_CRC5+0x4f, 0x11);
	HW_WR_REG8(LUT_CRC5+0x50, 0x1d);
	HW_WR_REG8(LUT_CRC5+0x51, 0x18);
	HW_WR_REG8(LUT_CRC5+0x52, 0x17);
	HW_WR_REG8(LUT_CRC5+0x53, 0x12);
	HW_WR_REG8(LUT_CRC5+0x54, 0x9);
	HW_WR_REG8(LUT_CRC5+0x55, 0xc);
	HW_WR_REG8(LUT_CRC5+0x56, 0x3);
	HW_WR_REG8(LUT_CRC5+0x57, 0x6);
	HW_WR_REG8(LUT_CRC5+0x58, 0x10);
	HW_WR_REG8(LUT_CRC5+0x59, 0x15);
	HW_WR_REG8(LUT_CRC5+0x5a, 0x1a);
	HW_WR_REG8(LUT_CRC5+0x5b, 0x1f);
	HW_WR_REG8(LUT_CRC5+0x5c, 0x4);
	HW_WR_REG8(LUT_CRC5+0x5d, 0x1);
	HW_WR_REG8(LUT_CRC5+0x5e, 0xe);
	HW_WR_REG8(LUT_CRC5+0x5f, 0xb);
	HW_WR_REG8(LUT_CRC5+0x60, 0x16);
	HW_WR_REG8(LUT_CRC5+0x61, 0x13);
	HW_WR_REG8(LUT_CRC5+0x62, 0x1c);
	HW_WR_REG8(LUT_CRC5+0x63, 0x19);
	HW_WR_REG8(LUT_CRC5+0x64, 0x2);
	HW_WR_REG8(LUT_CRC5+0x65, 0x7);
	HW_WR_REG8(LUT_CRC5+0x66, 0x8);
	HW_WR_REG8(LUT_CRC5+0x67, 0xd);
	HW_WR_REG8(LUT_CRC5+0x68, 0x1b);
	HW_WR_REG8(LUT_CRC5+0x69, 0x1e);
	HW_WR_REG8(LUT_CRC5+0x6a, 0x11);
	HW_WR_REG8(LUT_CRC5+0x6b, 0x14);
	HW_WR_REG8(LUT_CRC5+0x6c, 0xf);
	HW_WR_REG8(LUT_CRC5+0x6d, 0xa);
	HW_WR_REG8(LUT_CRC5+0x6e, 0x5);
	HW_WR_REG8(LUT_CRC5+0x6f, 0x0);
	HW_WR_REG8(LUT_CRC5+0x70, 0xc);
	HW_WR_REG8(LUT_CRC5+0x71, 0x9);
	HW_WR_REG8(LUT_CRC5+0x72, 0x6);
	HW_WR_REG8(LUT_CRC5+0x73, 0x3);
	HW_WR_REG8(LUT_CRC5+0x74, 0x18);
	HW_WR_REG8(LUT_CRC5+0x75, 0x1d);
	HW_WR_REG8(LUT_CRC5+0x76, 0x12);
	HW_WR_REG8(LUT_CRC5+0x77, 0x17);
	HW_WR_REG8(LUT_CRC5+0x78, 0x1);
	HW_WR_REG8(LUT_CRC5+0x79, 0x4);
	HW_WR_REG8(LUT_CRC5+0x7a, 0xb);
	HW_WR_REG8(LUT_CRC5+0x7b, 0xe);
	HW_WR_REG8(LUT_CRC5+0x7c, 0x15);
	HW_WR_REG8(LUT_CRC5+0x7d, 0x10);
	HW_WR_REG8(LUT_CRC5+0x7e, 0x1f);
	HW_WR_REG8(LUT_CRC5+0x7f, 0x1a);
	HW_WR_REG8(LUT_CRC5+0x80, 0xe);
	HW_WR_REG8(LUT_CRC5+0x81, 0xb);
	HW_WR_REG8(LUT_CRC5+0x82, 0x4);
	HW_WR_REG8(LUT_CRC5+0x83, 0x1);
	HW_WR_REG8(LUT_CRC5+0x84, 0x1a);
	HW_WR_REG8(LUT_CRC5+0x85, 0x1f);
	HW_WR_REG8(LUT_CRC5+0x86, 0x10);
	HW_WR_REG8(LUT_CRC5+0x87, 0x15);
	HW_WR_REG8(LUT_CRC5+0x88, 0x3);
	HW_WR_REG8(LUT_CRC5+0x89, 0x6);
	HW_WR_REG8(LUT_CRC5+0x8a, 0x9);
	HW_WR_REG8(LUT_CRC5+0x8b, 0xc);
	HW_WR_REG8(LUT_CRC5+0x8c, 0x17);
	HW_WR_REG8(LUT_CRC5+0x8d, 0x12);
	HW_WR_REG8(LUT_CRC5+0x8e, 0x1d);
	HW_WR_REG8(LUT_CRC5+0x8f, 0x18);
	HW_WR_REG8(LUT_CRC5+0x90, 0x14);
	HW_WR_REG8(LUT_CRC5+0x91, 0x11);
	HW_WR_REG8(LUT_CRC5+0x92, 0x1e);
	HW_WR_REG8(LUT_CRC5+0x93, 0x1b);
	HW_WR_REG8(LUT_CRC5+0x94, 0x0);
	HW_WR_REG8(LUT_CRC5+0x95, 0x5);
	HW_WR_REG8(LUT_CRC5+0x96, 0xa);
	HW_WR_REG8(LUT_CRC5+0x97, 0xf);
	HW_WR_REG8(LUT_CRC5+0x98, 0x19);
	HW_WR_REG8(LUT_CRC5+0x99, 0x1c);
	HW_WR_REG8(LUT_CRC5+0x9a, 0x13);
	HW_WR_REG8(LUT_CRC5+0x9b, 0x16);
	HW_WR_REG8(LUT_CRC5+0x9c, 0xd);
	HW_WR_REG8(LUT_CRC5+0x9d, 0x8);
	HW_WR_REG8(LUT_CRC5+0x9e, 0x7);
	HW_WR_REG8(LUT_CRC5+0x9f, 0x2);
	HW_WR_REG8(LUT_CRC5+0xa0, 0x1f);
	HW_WR_REG8(LUT_CRC5+0xa1, 0x1a);
	HW_WR_REG8(LUT_CRC5+0xa2, 0x15);
	HW_WR_REG8(LUT_CRC5+0xa3, 0x10);
	HW_WR_REG8(LUT_CRC5+0xa4, 0xb);
	HW_WR_REG8(LUT_CRC5+0xa5, 0xe);
	HW_WR_REG8(LUT_CRC5+0xa6, 0x1);
	HW_WR_REG8(LUT_CRC5+0xa7, 0x4);
	HW_WR_REG8(LUT_CRC5+0xa8, 0x12);
	HW_WR_REG8(LUT_CRC5+0xa9, 0x17);
	HW_WR_REG8(LUT_CRC5+0xaa, 0x18);
	HW_WR_REG8(LUT_CRC5+0xab, 0x1d);
	HW_WR_REG8(LUT_CRC5+0xac, 0x6);
	HW_WR_REG8(LUT_CRC5+0xad, 0x3);
	HW_WR_REG8(LUT_CRC5+0xae, 0xc);
	HW_WR_REG8(LUT_CRC5+0xaf, 0x9);
	HW_WR_REG8(LUT_CRC5+0xb0, 0x5);
	HW_WR_REG8(LUT_CRC5+0xb1, 0x0);
	HW_WR_REG8(LUT_CRC5+0xb2, 0xf);
	HW_WR_REG8(LUT_CRC5+0xb3, 0xa);
	HW_WR_REG8(LUT_CRC5+0xb4, 0x11);
	HW_WR_REG8(LUT_CRC5+0xb5, 0x14);
	HW_WR_REG8(LUT_CRC5+0xb6, 0x1b);
	HW_WR_REG8(LUT_CRC5+0xb7, 0x1e);
	HW_WR_REG8(LUT_CRC5+0xb8, 0x8);
	HW_WR_REG8(LUT_CRC5+0xb9, 0xd);
	HW_WR_REG8(LUT_CRC5+0xba, 0x2);
	HW_WR_REG8(LUT_CRC5+0xbb, 0x7);
	HW_WR_REG8(LUT_CRC5+0xbc, 0x1c);
	HW_WR_REG8(LUT_CRC5+0xbd, 0x19);
	HW_WR_REG8(LUT_CRC5+0xbe, 0x16);
	HW_WR_REG8(LUT_CRC5+0xbf, 0x13);
	HW_WR_REG8(LUT_CRC5+0xc0, 0x9);
	HW_WR_REG8(LUT_CRC5+0xc1, 0xc);
	HW_WR_REG8(LUT_CRC5+0xc2, 0x3);
	HW_WR_REG8(LUT_CRC5+0xc3, 0x6);
	HW_WR_REG8(LUT_CRC5+0xc4, 0x1d);
	HW_WR_REG8(LUT_CRC5+0xc5, 0x18);
	HW_WR_REG8(LUT_CRC5+0xc6, 0x17);
	HW_WR_REG8(LUT_CRC5+0xc7, 0x12);
	HW_WR_REG8(LUT_CRC5+0xc8, 0x4);
	HW_WR_REG8(LUT_CRC5+0xc9, 0x1);
	HW_WR_REG8(LUT_CRC5+0xca, 0xe);
	HW_WR_REG8(LUT_CRC5+0xcb, 0xb);
	HW_WR_REG8(LUT_CRC5+0xcc, 0x10);
	HW_WR_REG8(LUT_CRC5+0xcd, 0x15);
	HW_WR_REG8(LUT_CRC5+0xce, 0x1a);
	HW_WR_REG8(LUT_CRC5+0xcf, 0x1f);
	HW_WR_REG8(LUT_CRC5+0xd0, 0x13);
	HW_WR_REG8(LUT_CRC5+0xd1, 0x16);
	HW_WR_REG8(LUT_CRC5+0xd2, 0x19);
	HW_WR_REG8(LUT_CRC5+0xd3, 0x1c);
	HW_WR_REG8(LUT_CRC5+0xd4, 0x7);
	HW_WR_REG8(LUT_CRC5+0xd5, 0x2);
	HW_WR_REG8(LUT_CRC5+0xd6, 0xd);
	HW_WR_REG8(LUT_CRC5+0xd7, 0x8);
	HW_WR_REG8(LUT_CRC5+0xd8, 0x1e);
	HW_WR_REG8(LUT_CRC5+0xd9, 0x1b);
	HW_WR_REG8(LUT_CRC5+0xda, 0x14);
	HW_WR_REG8(LUT_CRC5+0xdb, 0x11);
	HW_WR_REG8(LUT_CRC5+0xdc, 0xa);
	HW_WR_REG8(LUT_CRC5+0xdd, 0xf);
	HW_WR_REG8(LUT_CRC5+0xde, 0x0);
	HW_WR_REG8(LUT_CRC5+0xdf, 0x5);
	HW_WR_REG8(LUT_CRC5+0xe0, 0x18);
	HW_WR_REG8(LUT_CRC5+0xe1, 0x1d);
	HW_WR_REG8(LUT_CRC5+0xe2, 0x12);
	HW_WR_REG8(LUT_CRC5+0xe3, 0x17);
	HW_WR_REG8(LUT_CRC5+0xe4, 0xc);
	HW_WR_REG8(LUT_CRC5+0xe5, 0x9);
	HW_WR_REG8(LUT_CRC5+0xe6, 0x6);
	HW_WR_REG8(LUT_CRC5+0xe7, 0x3);
	HW_WR_REG8(LUT_CRC5+0xe8, 0x15);
	HW_WR_REG8(LUT_CRC5+0xe9, 0x10);
	HW_WR_REG8(LUT_CRC5+0xea, 0x1f);
	HW_WR_REG8(LUT_CRC5+0xeb, 0x1a);
	HW_WR_REG8(LUT_CRC5+0xec, 0x1);
	HW_WR_REG8(LUT_CRC5+0xed, 0x4);
	HW_WR_REG8(LUT_CRC5+0xee, 0xb);
	HW_WR_REG8(LUT_CRC5+0xef, 0xe);
	HW_WR_REG8(LUT_CRC5+0xf0, 0x2);
	HW_WR_REG8(LUT_CRC5+0xf1, 0x7);
	HW_WR_REG8(LUT_CRC5+0xf2, 0x8);
	HW_WR_REG8(LUT_CRC5+0xf3, 0xd);
	HW_WR_REG8(LUT_CRC5+0xf4, 0x16);
	HW_WR_REG8(LUT_CRC5+0xf5, 0x13);
	HW_WR_REG8(LUT_CRC5+0xf6, 0x1c);
	HW_WR_REG8(LUT_CRC5+0xf7, 0x19);
	HW_WR_REG8(LUT_CRC5+0xf8, 0xf);
	HW_WR_REG8(LUT_CRC5+0xf9, 0xa);
	HW_WR_REG8(LUT_CRC5+0xfa, 0x5);
	HW_WR_REG8(LUT_CRC5+0xfb, 0x0);
	HW_WR_REG8(LUT_CRC5+0xfc, 0x1b);
	HW_WR_REG8(LUT_CRC5+0xfd, 0x1e);
	HW_WR_REG8(LUT_CRC5+0xfe, 0x11);
	HW_WR_REG8(LUT_CRC5+0xff, 0x14);
}

/**
 * @brief CRC configuration function
 *
 *
 */
void GenerateCRC16LUT(void)
{
	HW_WR_REG16(LUT_CRC16+0x0, 0x0);
	HW_WR_REG16(LUT_CRC16+0x2, 0x90d9);
	HW_WR_REG16(LUT_CRC16+0x4, 0xb16b);
	HW_WR_REG16(LUT_CRC16+0x6, 0x21b2);
	HW_WR_REG16(LUT_CRC16+0x8, 0xf20f);
	HW_WR_REG16(LUT_CRC16+0xa, 0x62d6);
	HW_WR_REG16(LUT_CRC16+0xc, 0x4364);
	HW_WR_REG16(LUT_CRC16+0xe, 0xd3bd);
	HW_WR_REG16(LUT_CRC16+0x10, 0x74c7);
	HW_WR_REG16(LUT_CRC16+0x12, 0xe41e);
	HW_WR_REG16(LUT_CRC16+0x14, 0xc5ac);
	HW_WR_REG16(LUT_CRC16+0x16, 0x5575);
	HW_WR_REG16(LUT_CRC16+0x18, 0x86c8);
	HW_WR_REG16(LUT_CRC16+0x1a, 0x1611);
	HW_WR_REG16(LUT_CRC16+0x1c, 0x37a3);
	HW_WR_REG16(LUT_CRC16+0x1e, 0xa77a);
	HW_WR_REG16(LUT_CRC16+0x20, 0xe98e);
	HW_WR_REG16(LUT_CRC16+0x22, 0x7957);
	HW_WR_REG16(LUT_CRC16+0x24, 0x58e5);
	HW_WR_REG16(LUT_CRC16+0x26, 0xc83c);
	HW_WR_REG16(LUT_CRC16+0x28, 0x1b81);
	HW_WR_REG16(LUT_CRC16+0x2a, 0x8b58);
	HW_WR_REG16(LUT_CRC16+0x2c, 0xaaea);
	HW_WR_REG16(LUT_CRC16+0x2e, 0x3a33);
	HW_WR_REG16(LUT_CRC16+0x30, 0x9d49);
	HW_WR_REG16(LUT_CRC16+0x32, 0xd90);
	HW_WR_REG16(LUT_CRC16+0x34, 0x2c22);
	HW_WR_REG16(LUT_CRC16+0x36, 0xbcfb);
	HW_WR_REG16(LUT_CRC16+0x38, 0x6f46);
	HW_WR_REG16(LUT_CRC16+0x3a, 0xff9f);
	HW_WR_REG16(LUT_CRC16+0x3c, 0xde2d);
	HW_WR_REG16(LUT_CRC16+0x3e, 0x4ef4);
	HW_WR_REG16(LUT_CRC16+0x40, 0x43c5);
	HW_WR_REG16(LUT_CRC16+0x42, 0xd31c);
	HW_WR_REG16(LUT_CRC16+0x44, 0xf2ae);
	HW_WR_REG16(LUT_CRC16+0x46, 0x6277);
	HW_WR_REG16(LUT_CRC16+0x48, 0xb1ca);
	HW_WR_REG16(LUT_CRC16+0x4a, 0x2113);
	HW_WR_REG16(LUT_CRC16+0x4c, 0xa1);
	HW_WR_REG16(LUT_CRC16+0x4e, 0x9078);
	HW_WR_REG16(LUT_CRC16+0x50, 0x3702);
	HW_WR_REG16(LUT_CRC16+0x52, 0xa7db);
	HW_WR_REG16(LUT_CRC16+0x54, 0x8669);
	HW_WR_REG16(LUT_CRC16+0x56, 0x16b0);
	HW_WR_REG16(LUT_CRC16+0x58, 0xc50d);
	HW_WR_REG16(LUT_CRC16+0x5a, 0x55d4);
	HW_WR_REG16(LUT_CRC16+0x5c, 0x7466);
	HW_WR_REG16(LUT_CRC16+0x5e, 0xe4bf);
	HW_WR_REG16(LUT_CRC16+0x60, 0xaa4b);
	HW_WR_REG16(LUT_CRC16+0x62, 0x3a92);
	HW_WR_REG16(LUT_CRC16+0x64, 0x1b20);
	HW_WR_REG16(LUT_CRC16+0x66, 0x8bf9);
	HW_WR_REG16(LUT_CRC16+0x68, 0x5844);
	HW_WR_REG16(LUT_CRC16+0x6a, 0xc89d);
	HW_WR_REG16(LUT_CRC16+0x6c, 0xe92f);
	HW_WR_REG16(LUT_CRC16+0x6e, 0x79f6);
	HW_WR_REG16(LUT_CRC16+0x70, 0xde8c);
	HW_WR_REG16(LUT_CRC16+0x72, 0x4e55);
	HW_WR_REG16(LUT_CRC16+0x74, 0x6fe7);
	HW_WR_REG16(LUT_CRC16+0x76, 0xff3e);
	HW_WR_REG16(LUT_CRC16+0x78, 0x2c83);
	HW_WR_REG16(LUT_CRC16+0x7a, 0xbc5a);
	HW_WR_REG16(LUT_CRC16+0x7c, 0x9de8);
	HW_WR_REG16(LUT_CRC16+0x7e, 0xd31);
	HW_WR_REG16(LUT_CRC16+0x80, 0x878a);
	HW_WR_REG16(LUT_CRC16+0x82, 0x1753);
	HW_WR_REG16(LUT_CRC16+0x84, 0x36e1);
	HW_WR_REG16(LUT_CRC16+0x86, 0xa638);
	HW_WR_REG16(LUT_CRC16+0x88, 0x7585);
	HW_WR_REG16(LUT_CRC16+0x8a, 0xe55c);
	HW_WR_REG16(LUT_CRC16+0x8c, 0xc4ee);
	HW_WR_REG16(LUT_CRC16+0x8e, 0x5437);
	HW_WR_REG16(LUT_CRC16+0x90, 0xf34d);
	HW_WR_REG16(LUT_CRC16+0x92, 0x6394);
	HW_WR_REG16(LUT_CRC16+0x94, 0x4226);
	HW_WR_REG16(LUT_CRC16+0x96, 0xd2ff);
	HW_WR_REG16(LUT_CRC16+0x98, 0x142);
	HW_WR_REG16(LUT_CRC16+0x9a, 0x919b);
	HW_WR_REG16(LUT_CRC16+0x9c, 0xb029);
	HW_WR_REG16(LUT_CRC16+0x9e, 0x20f0);
	HW_WR_REG16(LUT_CRC16+0xa0, 0x6e04);
	HW_WR_REG16(LUT_CRC16+0xa2, 0xfedd);
	HW_WR_REG16(LUT_CRC16+0xa4, 0xdf6f);
	HW_WR_REG16(LUT_CRC16+0xa6, 0x4fb6);
	HW_WR_REG16(LUT_CRC16+0xa8, 0x9c0b);
	HW_WR_REG16(LUT_CRC16+0xaa, 0xcd2);
	HW_WR_REG16(LUT_CRC16+0xac, 0x2d60);
	HW_WR_REG16(LUT_CRC16+0xae, 0xbdb9);
	HW_WR_REG16(LUT_CRC16+0xb0, 0x1ac3);
	HW_WR_REG16(LUT_CRC16+0xb2, 0x8a1a);
	HW_WR_REG16(LUT_CRC16+0xb4, 0xaba8);
	HW_WR_REG16(LUT_CRC16+0xb6, 0x3b71);
	HW_WR_REG16(LUT_CRC16+0xb8, 0xe8cc);
	HW_WR_REG16(LUT_CRC16+0xba, 0x7815);
	HW_WR_REG16(LUT_CRC16+0xbc, 0x59a7);
	HW_WR_REG16(LUT_CRC16+0xbe, 0xc97e);
	HW_WR_REG16(LUT_CRC16+0xc0, 0xc44f);
	HW_WR_REG16(LUT_CRC16+0xc2, 0x5496);
	HW_WR_REG16(LUT_CRC16+0xc4, 0x7524);
	HW_WR_REG16(LUT_CRC16+0xc6, 0xe5fd);
	HW_WR_REG16(LUT_CRC16+0xc8, 0x3640);
	HW_WR_REG16(LUT_CRC16+0xca, 0xa699);
	HW_WR_REG16(LUT_CRC16+0xcc, 0x872b);
	HW_WR_REG16(LUT_CRC16+0xce, 0x17f2);
	HW_WR_REG16(LUT_CRC16+0xd0, 0xb088);
	HW_WR_REG16(LUT_CRC16+0xd2, 0x2051);
	HW_WR_REG16(LUT_CRC16+0xd4, 0x1e3);
	HW_WR_REG16(LUT_CRC16+0xd6, 0x913a);
	HW_WR_REG16(LUT_CRC16+0xd8, 0x4287);
	HW_WR_REG16(LUT_CRC16+0xda, 0xd25e);
	HW_WR_REG16(LUT_CRC16+0xdc, 0xf3ec);
	HW_WR_REG16(LUT_CRC16+0xde, 0x6335);
	HW_WR_REG16(LUT_CRC16+0xe0, 0x2dc1);
	HW_WR_REG16(LUT_CRC16+0xe2, 0xbd18);
	HW_WR_REG16(LUT_CRC16+0xe4, 0x9caa);
	HW_WR_REG16(LUT_CRC16+0xe6, 0xc73);
	HW_WR_REG16(LUT_CRC16+0xe8, 0xdfce);
	HW_WR_REG16(LUT_CRC16+0xea, 0x4f17);
	HW_WR_REG16(LUT_CRC16+0xec, 0x6ea5);
	HW_WR_REG16(LUT_CRC16+0xee, 0xfe7c);
	HW_WR_REG16(LUT_CRC16+0xf0, 0x5906);
	HW_WR_REG16(LUT_CRC16+0xf2, 0xc9df);
	HW_WR_REG16(LUT_CRC16+0xf4, 0xe86d);
	HW_WR_REG16(LUT_CRC16+0xf6, 0x78b4);
	HW_WR_REG16(LUT_CRC16+0xf8, 0xab09);
	HW_WR_REG16(LUT_CRC16+0xfa, 0x3bd0);
	HW_WR_REG16(LUT_CRC16+0xfc, 0x1a62);
	HW_WR_REG16(LUT_CRC16+0xfe, 0x8abb);
	HW_WR_REG16(LUT_CRC16+0x100, 0x9fcd);
	HW_WR_REG16(LUT_CRC16+0x102, 0xf14);
	HW_WR_REG16(LUT_CRC16+0x104, 0x2ea6);
	HW_WR_REG16(LUT_CRC16+0x106, 0xbe7f);
	HW_WR_REG16(LUT_CRC16+0x108, 0x6dc2);
	HW_WR_REG16(LUT_CRC16+0x10a, 0xfd1b);
	HW_WR_REG16(LUT_CRC16+0x10c, 0xdca9);
	HW_WR_REG16(LUT_CRC16+0x10e, 0x4c70);
	HW_WR_REG16(LUT_CRC16+0x110, 0xeb0a);
	HW_WR_REG16(LUT_CRC16+0x112, 0x7bd3);
	HW_WR_REG16(LUT_CRC16+0x114, 0x5a61);
	HW_WR_REG16(LUT_CRC16+0x116, 0xcab8);
	HW_WR_REG16(LUT_CRC16+0x118, 0x1905);
	HW_WR_REG16(LUT_CRC16+0x11a, 0x89dc);
	HW_WR_REG16(LUT_CRC16+0x11c, 0xa86e);
	HW_WR_REG16(LUT_CRC16+0x11e, 0x38b7);
	HW_WR_REG16(LUT_CRC16+0x120, 0x7643);
	HW_WR_REG16(LUT_CRC16+0x122, 0xe69a);
	HW_WR_REG16(LUT_CRC16+0x124, 0xc728);
	HW_WR_REG16(LUT_CRC16+0x126, 0x57f1);
	HW_WR_REG16(LUT_CRC16+0x128, 0x844c);
	HW_WR_REG16(LUT_CRC16+0x12a, 0x1495);
	HW_WR_REG16(LUT_CRC16+0x12c, 0x3527);
	HW_WR_REG16(LUT_CRC16+0x12e, 0xa5fe);
	HW_WR_REG16(LUT_CRC16+0x130, 0x284);
	HW_WR_REG16(LUT_CRC16+0x132, 0x925d);
	HW_WR_REG16(LUT_CRC16+0x134, 0xb3ef);
	HW_WR_REG16(LUT_CRC16+0x136, 0x2336);
	HW_WR_REG16(LUT_CRC16+0x138, 0xf08b);
	HW_WR_REG16(LUT_CRC16+0x13a, 0x6052);
	HW_WR_REG16(LUT_CRC16+0x13c, 0x41e0);
	HW_WR_REG16(LUT_CRC16+0x13e, 0xd139);
	HW_WR_REG16(LUT_CRC16+0x140, 0xdc08);
	HW_WR_REG16(LUT_CRC16+0x142, 0x4cd1);
	HW_WR_REG16(LUT_CRC16+0x144, 0x6d63);
	HW_WR_REG16(LUT_CRC16+0x146, 0xfdba);
	HW_WR_REG16(LUT_CRC16+0x148, 0x2e07);
	HW_WR_REG16(LUT_CRC16+0x14a, 0xbede);
	HW_WR_REG16(LUT_CRC16+0x14c, 0x9f6c);
	HW_WR_REG16(LUT_CRC16+0x14e, 0xfb5);
	HW_WR_REG16(LUT_CRC16+0x150, 0xa8cf);
	HW_WR_REG16(LUT_CRC16+0x152, 0x3816);
	HW_WR_REG16(LUT_CRC16+0x154, 0x19a4);
	HW_WR_REG16(LUT_CRC16+0x156, 0x897d);
	HW_WR_REG16(LUT_CRC16+0x158, 0x5ac0);
	HW_WR_REG16(LUT_CRC16+0x15a, 0xca19);
	HW_WR_REG16(LUT_CRC16+0x15c, 0xebab);
	HW_WR_REG16(LUT_CRC16+0x15e, 0x7b72);
	HW_WR_REG16(LUT_CRC16+0x160, 0x3586);
	HW_WR_REG16(LUT_CRC16+0x162, 0xa55f);
	HW_WR_REG16(LUT_CRC16+0x164, 0x84ed);
	HW_WR_REG16(LUT_CRC16+0x166, 0x1434);
	HW_WR_REG16(LUT_CRC16+0x168, 0xc789);
	HW_WR_REG16(LUT_CRC16+0x16a, 0x5750);
	HW_WR_REG16(LUT_CRC16+0x16c, 0x76e2);
	HW_WR_REG16(LUT_CRC16+0x16e, 0xe63b);
	HW_WR_REG16(LUT_CRC16+0x170, 0x4141);
	HW_WR_REG16(LUT_CRC16+0x172, 0xd198);
	HW_WR_REG16(LUT_CRC16+0x174, 0xf02a);
	HW_WR_REG16(LUT_CRC16+0x176, 0x60f3);
	HW_WR_REG16(LUT_CRC16+0x178, 0xb34e);
	HW_WR_REG16(LUT_CRC16+0x17a, 0x2397);
	HW_WR_REG16(LUT_CRC16+0x17c, 0x225);
	HW_WR_REG16(LUT_CRC16+0x17e, 0x92fc);
	HW_WR_REG16(LUT_CRC16+0x180, 0x1847);
	HW_WR_REG16(LUT_CRC16+0x182, 0x889e);
	HW_WR_REG16(LUT_CRC16+0x184, 0xa92c);
	HW_WR_REG16(LUT_CRC16+0x186, 0x39f5);
	HW_WR_REG16(LUT_CRC16+0x188, 0xea48);
	HW_WR_REG16(LUT_CRC16+0x18a, 0x7a91);
	HW_WR_REG16(LUT_CRC16+0x18c, 0x5b23);
	HW_WR_REG16(LUT_CRC16+0x18e, 0xcbfa);
	HW_WR_REG16(LUT_CRC16+0x190, 0x6c80);
	HW_WR_REG16(LUT_CRC16+0x192, 0xfc59);
	HW_WR_REG16(LUT_CRC16+0x194, 0xddeb);
	HW_WR_REG16(LUT_CRC16+0x196, 0x4d32);
	HW_WR_REG16(LUT_CRC16+0x198, 0x9e8f);
	HW_WR_REG16(LUT_CRC16+0x19a, 0xe56);
	HW_WR_REG16(LUT_CRC16+0x19c, 0x2fe4);
	HW_WR_REG16(LUT_CRC16+0x19e, 0xbf3d);
	HW_WR_REG16(LUT_CRC16+0x1a0, 0xf1c9);
	HW_WR_REG16(LUT_CRC16+0x1a2, 0x6110);
	HW_WR_REG16(LUT_CRC16+0x1a4, 0x40a2);
	HW_WR_REG16(LUT_CRC16+0x1a6, 0xd07b);
	HW_WR_REG16(LUT_CRC16+0x1a8, 0x3c6);
	HW_WR_REG16(LUT_CRC16+0x1aa, 0x931f);
	HW_WR_REG16(LUT_CRC16+0x1ac, 0xb2ad);
	HW_WR_REG16(LUT_CRC16+0x1ae, 0x2274);
	HW_WR_REG16(LUT_CRC16+0x1b0, 0x850e);
	HW_WR_REG16(LUT_CRC16+0x1b2, 0x15d7);
	HW_WR_REG16(LUT_CRC16+0x1b4, 0x3465);
	HW_WR_REG16(LUT_CRC16+0x1b6, 0xa4bc);
	HW_WR_REG16(LUT_CRC16+0x1b8, 0x7701);
	HW_WR_REG16(LUT_CRC16+0x1ba, 0xe7d8);
	HW_WR_REG16(LUT_CRC16+0x1bc, 0xc66a);
	HW_WR_REG16(LUT_CRC16+0x1be, 0x56b3);
	HW_WR_REG16(LUT_CRC16+0x1c0, 0x5b82);
	HW_WR_REG16(LUT_CRC16+0x1c2, 0xcb5b);
	HW_WR_REG16(LUT_CRC16+0x1c4, 0xeae9);
	HW_WR_REG16(LUT_CRC16+0x1c6, 0x7a30);
	HW_WR_REG16(LUT_CRC16+0x1c8, 0xa98d);
	HW_WR_REG16(LUT_CRC16+0x1ca, 0x3954);
	HW_WR_REG16(LUT_CRC16+0x1cc, 0x18e6);
	HW_WR_REG16(LUT_CRC16+0x1ce, 0x883f);
	HW_WR_REG16(LUT_CRC16+0x1d0, 0x2f45);
	HW_WR_REG16(LUT_CRC16+0x1d2, 0xbf9c);
	HW_WR_REG16(LUT_CRC16+0x1d4, 0x9e2e);
	HW_WR_REG16(LUT_CRC16+0x1d6, 0xef7);
	HW_WR_REG16(LUT_CRC16+0x1d8, 0xdd4a);
	HW_WR_REG16(LUT_CRC16+0x1da, 0x4d93);
	HW_WR_REG16(LUT_CRC16+0x1dc, 0x6c21);
	HW_WR_REG16(LUT_CRC16+0x1de, 0xfcf8);
	HW_WR_REG16(LUT_CRC16+0x1e0, 0xb20c);
	HW_WR_REG16(LUT_CRC16+0x1e2, 0x22d5);
	HW_WR_REG16(LUT_CRC16+0x1e4, 0x367);
	HW_WR_REG16(LUT_CRC16+0x1e6, 0x93be);
	HW_WR_REG16(LUT_CRC16+0x1e8, 0x4003);
	HW_WR_REG16(LUT_CRC16+0x1ea, 0xd0da);
	HW_WR_REG16(LUT_CRC16+0x1ec, 0xf168);
	HW_WR_REG16(LUT_CRC16+0x1ee, 0x61b1);
	HW_WR_REG16(LUT_CRC16+0x1f0, 0xc6cb);
	HW_WR_REG16(LUT_CRC16+0x1f2, 0x5612);
	HW_WR_REG16(LUT_CRC16+0x1f4, 0x77a0);
	HW_WR_REG16(LUT_CRC16+0x1f6, 0xe779);
	HW_WR_REG16(LUT_CRC16+0x1f8, 0x34c4);
	HW_WR_REG16(LUT_CRC16+0x1fa, 0xa41d);
	HW_WR_REG16(LUT_CRC16+0x1fc, 0x85af);
	HW_WR_REG16(LUT_CRC16+0x1fe, 0x1576);
}

/**
 * @brief helper configuration function
 *
 *
 */
void GenerateBitToByteLUT(void)
{
	HW_WR_REG32(LUT_BITTOBYTE+0x0, 0x0);
	HW_WR_REG32(LUT_BITTOBYTE+0x4, 0x1);
	HW_WR_REG32(LUT_BITTOBYTE+0x8, 0x100);
	HW_WR_REG32(LUT_BITTOBYTE+0xc, 0x101);
	HW_WR_REG32(LUT_BITTOBYTE+0x10, 0x10000);
	HW_WR_REG32(LUT_BITTOBYTE+0x14, 0x10001);
	HW_WR_REG32(LUT_BITTOBYTE+0x18, 0x10100);
	HW_WR_REG32(LUT_BITTOBYTE+0x1c, 0x10101);
	HW_WR_REG32(LUT_BITTOBYTE+0x20, 0x1000000);
	HW_WR_REG32(LUT_BITTOBYTE+0x24, 0x1000001);
	HW_WR_REG32(LUT_BITTOBYTE+0x28, 0x1000100);
	HW_WR_REG32(LUT_BITTOBYTE+0x2c, 0x1000101);
	HW_WR_REG32(LUT_BITTOBYTE+0x30, 0x1010000);
	HW_WR_REG32(LUT_BITTOBYTE+0x34, 0x1010001);
	HW_WR_REG32(LUT_BITTOBYTE+0x38, 0x1010100);
	HW_WR_REG32(LUT_BITTOBYTE+0x3c, 0x1010101);
}

/**
 * @brief look up table configuration function
 *
 *
 */
void GenerateRSSILUT(void)
{
	HW_WR_REG8(LUT_RSSI+0x0, 0xc);
	HW_WR_REG8(LUT_RSSI+0x1, 0xc);
	HW_WR_REG8(LUT_RSSI+0x2, 0xc);
	HW_WR_REG8(LUT_RSSI+0x3, 0xa);
	HW_WR_REG8(LUT_RSSI+0x4, 0xc);
	HW_WR_REG8(LUT_RSSI+0x5, 0x8);
	HW_WR_REG8(LUT_RSSI+0x6, 0xa);
	HW_WR_REG8(LUT_RSSI+0x7, 0x8);
	HW_WR_REG8(LUT_RSSI+0x8, 0xc);
	HW_WR_REG8(LUT_RSSI+0x9, 0x6);
	HW_WR_REG8(LUT_RSSI+0xa, 0x8);
	HW_WR_REG8(LUT_RSSI+0xb, 0x6);
	HW_WR_REG8(LUT_RSSI+0xc, 0xa);
	HW_WR_REG8(LUT_RSSI+0xd, 0x6);
	HW_WR_REG8(LUT_RSSI+0xe, 0x8);
	HW_WR_REG8(LUT_RSSI+0xf, 0x6);
	HW_WR_REG8(LUT_RSSI+0x10, 0xc);
	HW_WR_REG8(LUT_RSSI+0x11, 0x4);
	HW_WR_REG8(LUT_RSSI+0x12, 0x6);
	HW_WR_REG8(LUT_RSSI+0x13, 0x4);
	HW_WR_REG8(LUT_RSSI+0x14, 0x8);
	HW_WR_REG8(LUT_RSSI+0x15, 0x4);
	HW_WR_REG8(LUT_RSSI+0x16, 0x6);
	HW_WR_REG8(LUT_RSSI+0x17, 0x4);
	HW_WR_REG8(LUT_RSSI+0x18, 0xa);
	HW_WR_REG8(LUT_RSSI+0x19, 0x4);
	HW_WR_REG8(LUT_RSSI+0x1a, 0x6);
	HW_WR_REG8(LUT_RSSI+0x1b, 0x4);
	HW_WR_REG8(LUT_RSSI+0x1c, 0x8);
	HW_WR_REG8(LUT_RSSI+0x1d, 0x4);
	HW_WR_REG8(LUT_RSSI+0x1e, 0x6);
	HW_WR_REG8(LUT_RSSI+0x1f, 0x4);
	HW_WR_REG8(LUT_RSSI+0x20, 0xc);
	HW_WR_REG8(LUT_RSSI+0x21, 0x6);
	HW_WR_REG8(LUT_RSSI+0x22, 0x4);
	HW_WR_REG8(LUT_RSSI+0x23, 0x4);
	HW_WR_REG8(LUT_RSSI+0x24, 0x6);
	HW_WR_REG8(LUT_RSSI+0x25, 0x2);
	HW_WR_REG8(LUT_RSSI+0x26, 0x4);
	HW_WR_REG8(LUT_RSSI+0x27, 0x2);
	HW_WR_REG8(LUT_RSSI+0x28, 0x8);
	HW_WR_REG8(LUT_RSSI+0x29, 0x2);
	HW_WR_REG8(LUT_RSSI+0x2a, 0x4);
	HW_WR_REG8(LUT_RSSI+0x2b, 0x2);
	HW_WR_REG8(LUT_RSSI+0x2c, 0x6);
	HW_WR_REG8(LUT_RSSI+0x2d, 0x2);
	HW_WR_REG8(LUT_RSSI+0x2e, 0x4);
	HW_WR_REG8(LUT_RSSI+0x2f, 0x2);
	HW_WR_REG8(LUT_RSSI+0x30, 0xa);
	HW_WR_REG8(LUT_RSSI+0x31, 0x4);
	HW_WR_REG8(LUT_RSSI+0x32, 0x4);
	HW_WR_REG8(LUT_RSSI+0x33, 0x2);
	HW_WR_REG8(LUT_RSSI+0x34, 0x6);
	HW_WR_REG8(LUT_RSSI+0x35, 0x2);
	HW_WR_REG8(LUT_RSSI+0x36, 0x4);
	HW_WR_REG8(LUT_RSSI+0x37, 0x2);
	HW_WR_REG8(LUT_RSSI+0x38, 0x8);
	HW_WR_REG8(LUT_RSSI+0x39, 0x2);
	HW_WR_REG8(LUT_RSSI+0x3a, 0x4);
	HW_WR_REG8(LUT_RSSI+0x3b, 0x2);
	HW_WR_REG8(LUT_RSSI+0x3c, 0x6);
	HW_WR_REG8(LUT_RSSI+0x3d, 0x2);
	HW_WR_REG8(LUT_RSSI+0x3e, 0x4);
	HW_WR_REG8(LUT_RSSI+0x3f, 0x2);
	HW_WR_REG8(LUT_RSSI+0x40, 0xc);
	HW_WR_REG8(LUT_RSSI+0x41, 0x8);
	HW_WR_REG8(LUT_RSSI+0x42, 0x6);
	HW_WR_REG8(LUT_RSSI+0x43, 0x6);
	HW_WR_REG8(LUT_RSSI+0x44, 0x4);
	HW_WR_REG8(LUT_RSSI+0x45, 0x4);
	HW_WR_REG8(LUT_RSSI+0x46, 0x4);
	HW_WR_REG8(LUT_RSSI+0x47, 0x4);
	HW_WR_REG8(LUT_RSSI+0x48, 0x6);
	HW_WR_REG8(LUT_RSSI+0x49, 0x2);
	HW_WR_REG8(LUT_RSSI+0x4a, 0x2);
	HW_WR_REG8(LUT_RSSI+0x4b, 0x2);
	HW_WR_REG8(LUT_RSSI+0x4c, 0x4);
	HW_WR_REG8(LUT_RSSI+0x4d, 0x2);
	HW_WR_REG8(LUT_RSSI+0x4e, 0x2);
	HW_WR_REG8(LUT_RSSI+0x4f, 0x2);
	HW_WR_REG8(LUT_RSSI+0x50, 0x8);
	HW_WR_REG8(LUT_RSSI+0x51, 0x4);
	HW_WR_REG8(LUT_RSSI+0x52, 0x2);
	HW_WR_REG8(LUT_RSSI+0x53, 0x2);
	HW_WR_REG8(LUT_RSSI+0x54, 0x4);
	HW_WR_REG8(LUT_RSSI+0x55, 0xc);
	HW_WR_REG8(LUT_RSSI+0x56, 0x2);
	HW_WR_REG8(LUT_RSSI+0x57, 0xc);
	HW_WR_REG8(LUT_RSSI+0x58, 0x6);
	HW_WR_REG8(LUT_RSSI+0x59, 0x2);
	HW_WR_REG8(LUT_RSSI+0x5a, 0x2);
	HW_WR_REG8(LUT_RSSI+0x5b, 0xc);
	HW_WR_REG8(LUT_RSSI+0x5c, 0x4);
	HW_WR_REG8(LUT_RSSI+0x5d, 0xc);
	HW_WR_REG8(LUT_RSSI+0x5e, 0x2);
	HW_WR_REG8(LUT_RSSI+0x5f, 0xc);
	HW_WR_REG8(LUT_RSSI+0x60, 0xa);
	HW_WR_REG8(LUT_RSSI+0x61, 0x6);
	HW_WR_REG8(LUT_RSSI+0x62, 0x4);
	HW_WR_REG8(LUT_RSSI+0x63, 0x4);
	HW_WR_REG8(LUT_RSSI+0x64, 0x4);
	HW_WR_REG8(LUT_RSSI+0x65, 0x2);
	HW_WR_REG8(LUT_RSSI+0x66, 0x2);
	HW_WR_REG8(LUT_RSSI+0x67, 0x2);
	HW_WR_REG8(LUT_RSSI+0x68, 0x6);
	HW_WR_REG8(LUT_RSSI+0x69, 0x2);
	HW_WR_REG8(LUT_RSSI+0x6a, 0x2);
	HW_WR_REG8(LUT_RSSI+0x6b, 0xc);
	HW_WR_REG8(LUT_RSSI+0x6c, 0x4);
	HW_WR_REG8(LUT_RSSI+0x6d, 0xc);
	HW_WR_REG8(LUT_RSSI+0x6e, 0x2);
	HW_WR_REG8(LUT_RSSI+0x6f, 0xc);
	HW_WR_REG8(LUT_RSSI+0x70, 0x8);
	HW_WR_REG8(LUT_RSSI+0x71, 0x4);
	HW_WR_REG8(LUT_RSSI+0x72, 0x2);
	HW_WR_REG8(LUT_RSSI+0x73, 0x2);
	HW_WR_REG8(LUT_RSSI+0x74, 0x4);
	HW_WR_REG8(LUT_RSSI+0x75, 0xc);
	HW_WR_REG8(LUT_RSSI+0x76, 0x2);
	HW_WR_REG8(LUT_RSSI+0x77, 0xc);
	HW_WR_REG8(LUT_RSSI+0x78, 0x6);
	HW_WR_REG8(LUT_RSSI+0x79, 0x2);
	HW_WR_REG8(LUT_RSSI+0x7a, 0x2);
	HW_WR_REG8(LUT_RSSI+0x7b, 0xc);
	HW_WR_REG8(LUT_RSSI+0x7c, 0x4);
	HW_WR_REG8(LUT_RSSI+0x7d, 0xc);
	HW_WR_REG8(LUT_RSSI+0x7e, 0x2);
	HW_WR_REG8(LUT_RSSI+0x7f, 0xc);
	HW_WR_REG8(LUT_RSSI+0x80, 0xc);
	HW_WR_REG8(LUT_RSSI+0x81, 0xa);
	HW_WR_REG8(LUT_RSSI+0x82, 0x8);
	HW_WR_REG8(LUT_RSSI+0x83, 0x8);
	HW_WR_REG8(LUT_RSSI+0x84, 0x6);
	HW_WR_REG8(LUT_RSSI+0x85, 0x6);
	HW_WR_REG8(LUT_RSSI+0x86, 0x6);
	HW_WR_REG8(LUT_RSSI+0x87, 0x6);
	HW_WR_REG8(LUT_RSSI+0x88, 0x4);
	HW_WR_REG8(LUT_RSSI+0x89, 0x4);
	HW_WR_REG8(LUT_RSSI+0x8a, 0x4);
	HW_WR_REG8(LUT_RSSI+0x8b, 0x4);
	HW_WR_REG8(LUT_RSSI+0x8c, 0x4);
	HW_WR_REG8(LUT_RSSI+0x8d, 0x4);
	HW_WR_REG8(LUT_RSSI+0x8e, 0x4);
	HW_WR_REG8(LUT_RSSI+0x8f, 0x4);
	HW_WR_REG8(LUT_RSSI+0x90, 0x6);
	HW_WR_REG8(LUT_RSSI+0x91, 0x4);
	HW_WR_REG8(LUT_RSSI+0x92, 0x2);
	HW_WR_REG8(LUT_RSSI+0x93, 0x2);
	HW_WR_REG8(LUT_RSSI+0x94, 0x2);
	HW_WR_REG8(LUT_RSSI+0x95, 0x2);
	HW_WR_REG8(LUT_RSSI+0x96, 0x2);
	HW_WR_REG8(LUT_RSSI+0x97, 0x2);
	HW_WR_REG8(LUT_RSSI+0x98, 0x4);
	HW_WR_REG8(LUT_RSSI+0x99, 0x2);
	HW_WR_REG8(LUT_RSSI+0x9a, 0x2);
	HW_WR_REG8(LUT_RSSI+0x9b, 0x2);
	HW_WR_REG8(LUT_RSSI+0x9c, 0x2);
	HW_WR_REG8(LUT_RSSI+0x9d, 0x2);
	HW_WR_REG8(LUT_RSSI+0x9e, 0x2);
	HW_WR_REG8(LUT_RSSI+0x9f, 0x2);
	HW_WR_REG8(LUT_RSSI+0xa0, 0x8);
	HW_WR_REG8(LUT_RSSI+0xa1, 0x6);
	HW_WR_REG8(LUT_RSSI+0xa2, 0x4);
	HW_WR_REG8(LUT_RSSI+0xa3, 0x4);
	HW_WR_REG8(LUT_RSSI+0xa4, 0x2);
	HW_WR_REG8(LUT_RSSI+0xa5, 0x2);
	HW_WR_REG8(LUT_RSSI+0xa6, 0x2);
	HW_WR_REG8(LUT_RSSI+0xa7, 0x2);
	HW_WR_REG8(LUT_RSSI+0xa8, 0x4);
	HW_WR_REG8(LUT_RSSI+0xa9, 0x2);
	HW_WR_REG8(LUT_RSSI+0xaa, 0xc);
	HW_WR_REG8(LUT_RSSI+0xab, 0xc);
	HW_WR_REG8(LUT_RSSI+0xac, 0x2);
	HW_WR_REG8(LUT_RSSI+0xad, 0xc);
	HW_WR_REG8(LUT_RSSI+0xae, 0xc);
	HW_WR_REG8(LUT_RSSI+0xaf, 0xc);
	HW_WR_REG8(LUT_RSSI+0xb0, 0x6);
	HW_WR_REG8(LUT_RSSI+0xb1, 0x4);
	HW_WR_REG8(LUT_RSSI+0xb2, 0x2);
	HW_WR_REG8(LUT_RSSI+0xb3, 0x2);
	HW_WR_REG8(LUT_RSSI+0xb4, 0x2);
	HW_WR_REG8(LUT_RSSI+0xb5, 0xc);
	HW_WR_REG8(LUT_RSSI+0xb6, 0xc);
	HW_WR_REG8(LUT_RSSI+0xb7, 0xc);
	HW_WR_REG8(LUT_RSSI+0xb8, 0x4);
	HW_WR_REG8(LUT_RSSI+0xb9, 0x2);
	HW_WR_REG8(LUT_RSSI+0xba, 0xc);
	HW_WR_REG8(LUT_RSSI+0xbb, 0xc);
	HW_WR_REG8(LUT_RSSI+0xbc, 0x2);
	HW_WR_REG8(LUT_RSSI+0xbd, 0xc);
	HW_WR_REG8(LUT_RSSI+0xbe, 0xc);
	HW_WR_REG8(LUT_RSSI+0xbf, 0xc);
	HW_WR_REG8(LUT_RSSI+0xc0, 0xa);
	HW_WR_REG8(LUT_RSSI+0xc1, 0x8);
	HW_WR_REG8(LUT_RSSI+0xc2, 0x6);
	HW_WR_REG8(LUT_RSSI+0xc3, 0x6);
	HW_WR_REG8(LUT_RSSI+0xc4, 0x4);
	HW_WR_REG8(LUT_RSSI+0xc5, 0x4);
	HW_WR_REG8(LUT_RSSI+0xc6, 0x4);
	HW_WR_REG8(LUT_RSSI+0xc7, 0x4);
	HW_WR_REG8(LUT_RSSI+0xc8, 0x4);
	HW_WR_REG8(LUT_RSSI+0xc9, 0x2);
	HW_WR_REG8(LUT_RSSI+0xca, 0x2);
	HW_WR_REG8(LUT_RSSI+0xcb, 0x2);
	HW_WR_REG8(LUT_RSSI+0xcc, 0x2);
	HW_WR_REG8(LUT_RSSI+0xcd, 0x2);
	HW_WR_REG8(LUT_RSSI+0xce, 0x2);
	HW_WR_REG8(LUT_RSSI+0xcf, 0x2);
	HW_WR_REG8(LUT_RSSI+0xd0, 0x6);
	HW_WR_REG8(LUT_RSSI+0xd1, 0x4);
	HW_WR_REG8(LUT_RSSI+0xd2, 0x2);
	HW_WR_REG8(LUT_RSSI+0xd3, 0x2);
	HW_WR_REG8(LUT_RSSI+0xd4, 0x2);
	HW_WR_REG8(LUT_RSSI+0xd5, 0xc);
	HW_WR_REG8(LUT_RSSI+0xd6, 0xc);
	HW_WR_REG8(LUT_RSSI+0xd7, 0xc);
	HW_WR_REG8(LUT_RSSI+0xd8, 0x4);
	HW_WR_REG8(LUT_RSSI+0xd9, 0x2);
	HW_WR_REG8(LUT_RSSI+0xda, 0xc);
	HW_WR_REG8(LUT_RSSI+0xdb, 0xc);
	HW_WR_REG8(LUT_RSSI+0xdc, 0x2);
	HW_WR_REG8(LUT_RSSI+0xdd, 0xc);
	HW_WR_REG8(LUT_RSSI+0xde, 0xc);
	HW_WR_REG8(LUT_RSSI+0xdf, 0xc);
	HW_WR_REG8(LUT_RSSI+0xe0, 0x8);
	HW_WR_REG8(LUT_RSSI+0xe1, 0x6);
	HW_WR_REG8(LUT_RSSI+0xe2, 0x4);
	HW_WR_REG8(LUT_RSSI+0xe3, 0x4);
	HW_WR_REG8(LUT_RSSI+0xe4, 0x2);
	HW_WR_REG8(LUT_RSSI+0xe5, 0x2);
	HW_WR_REG8(LUT_RSSI+0xe6, 0x2);
	HW_WR_REG8(LUT_RSSI+0xe7, 0x2);
	HW_WR_REG8(LUT_RSSI+0xe8, 0x4);
	HW_WR_REG8(LUT_RSSI+0xe9, 0x2);
	HW_WR_REG8(LUT_RSSI+0xea, 0xc);
	HW_WR_REG8(LUT_RSSI+0xeb, 0xc);
	HW_WR_REG8(LUT_RSSI+0xec, 0x2);
	HW_WR_REG8(LUT_RSSI+0xed, 0xc);
	HW_WR_REG8(LUT_RSSI+0xee, 0xc);
	HW_WR_REG8(LUT_RSSI+0xef, 0xc);
	HW_WR_REG8(LUT_RSSI+0xf0, 0x6);
	HW_WR_REG8(LUT_RSSI+0xf1, 0x4);
	HW_WR_REG8(LUT_RSSI+0xf2, 0x2);
	HW_WR_REG8(LUT_RSSI+0xf3, 0x2);
	HW_WR_REG8(LUT_RSSI+0xf4, 0x2);
	HW_WR_REG8(LUT_RSSI+0xf5, 0xc);
	HW_WR_REG8(LUT_RSSI+0xf6, 0xc);
	HW_WR_REG8(LUT_RSSI+0xf7, 0xc);
	HW_WR_REG8(LUT_RSSI+0xf8, 0x4);
	HW_WR_REG8(LUT_RSSI+0xf9, 0x2);
	HW_WR_REG8(LUT_RSSI+0xfa, 0xc);
	HW_WR_REG8(LUT_RSSI+0xfb, 0xc);
	HW_WR_REG8(LUT_RSSI+0xfc, 0x2);
	HW_WR_REG8(LUT_RSSI+0xfd, 0xc);
	HW_WR_REG8(LUT_RSSI+0xfe, 0xc);
	HW_WR_REG8(LUT_RSSI+0xff, 0xc);
}

/**
 * @brief helper configuration function
 *
 *
 */
void GenerateEXTRAEDGELUT(void)
{
	HW_WR_REG8(LUT_EE+0, 0x00);
	HW_WR_REG8(LUT_EE+1, 0x80);
	HW_WR_REG8(LUT_EE+2, 0xc0);
	HW_WR_REG8(LUT_EE+3, 0xe0);
	HW_WR_REG8(LUT_EE+4, 0xf0);
	HW_WR_REG8(LUT_EE+5, 0xf8);
	HW_WR_REG8(LUT_EE+6, 0xfc);
	HW_WR_REG8(LUT_EE+7, 0xfe);
	HW_WR_REG8(LUT_EE+8, 0xff);
}

/**
 * @brief register interface configuration function
 *
 *
 */
void ConfigureRegisterIF(void)
{
	unsigned i;
	for(i = 0;i < MASTER_REGS_SIZE; i += 4)
		HW_WR_REG32(MASTER_REGS+i, 0);

	HW_WR_REG8(SYNC_CTRL, 1);
	HW_WR_REG8(PC_ADD_H, 0x80);
	HW_WR_REG8(PC_OFF_H, 0x80);
	HW_WR_REG16(MAXDEV_H, /*0xffff*/ 0);
	HW_WR_REG16(MAXDEV_THRESH_H, 0xffff);
	HW_WR_REG8(VERSION, 0x41);
	HW_WR_REG8(ACC_ERR_CNT_THRESH, 0x1f);

	HW_WR_REG32(TEST_PATTERN0, 0x4cc8115d);
	HW_WR_REG32(TEST_PATTERN0+4, 0xfb334990);
	HW_WR_REG32(TEST_PATTERN0+8, 0xfffff800);
}
