/*
* Multi-protocol digital position encoder master interface
*
* TI design to demonstrate the multi-protocol digital position encoder
* master interface for EnDat 2.2, Hiperface DSL and BiSS
*
* Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
*
*    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include <stdio.h>
#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>

#include "types.h"
#include "hw/hw_types.h"
#include <hw/hw_control_am43xx.h>
#include "osdrv_version.h"
#include "console_utils.h"
#include "osdrv_pruss.h"
#include "osdrv_utils.h"
#include "appl_cnfg.h"
#include "board_support.h"
#include "gpio.h"
#include "mpe_common.h"
#include "endat.h"
#include "hiperface_dsl.h"
#include "biss.h"

/** \brief PRU-ICSS LLD handle */
extern PRUICSS_Handle handle, pruIcss0Handle;

void ownReset(UArg, UArg);	// forward

int mpe_protocol_type = mpe_no_protocol;

boardId_t board_Type = BOARD_UNKNOWN;

#define APP_VERSION "v1.0"								// application version number
#define APP_NAME "Multi-Protocol Position Encoder Master Interface"

void MPEncoderTask(UArg arg0, UArg arg1);

/*
 * this runs just after reset before any BIOS stuff...
 * so here is the right place to enable/configure our hardware.
 * But we can't use any Bios functions yet!
 *
 */
void ownReset(UArg arg0, UArg arg1)
{
    /* Clock source selection */
      PRCMModuleEnable(CHIPDB_MOD_ID_GPIO, 0, FALSE);		// enable GPIO 0, 1 and 4
      PRCMModuleEnable(CHIPDB_MOD_ID_GPIO, 1, FALSE);
      PRCMModuleEnable(CHIPDB_MOD_ID_GPIO, 5, FALSE);
     PRCMModuleEnable(CHIPDB_MOD_ID_GPIO, 4, FALSE);
      // set GPIO directions
      GPIOSetDirMode(SOC_GPIO0_REG, 15, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO1_REG, 12, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO1_REG, 15, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO4_REG, 19, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO1_REG, 9, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO0_REG, 2, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO4_REG, 21, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO5_REG, 4, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO4_REG, 9, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO4_REG, 17, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO0_REG, 27, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO0_REG, 22, GPIO_DIRECTION_OUTPUT);
      GPIOSetDirMode(SOC_GPIO0_REG, 23, GPIO_DIRECTION_OUTPUT);
}

/**
 * \brief the application entry point - initializes low level hardware, creates tasks and starts SYSBIOS
 *
 */
Int main()
{
    Task_Params taskParams;

 	SDKMMUInit(applMmuEntries);
	BOARDInit(NULL);
    board_Type = BOARDGetId();

	Board_pinMuxConfig(am437xIdkMux);

    board_init(BOARD_TRICOLOR0_GREEN | BOARD_TRICOLOR1_YELLOW |
    		BOARD_TRICOLOR1_GREEN | BOARD_TRICOLOR1_RED | BOARD_TRICOLOR0_RED
			| BOARD_LED_DIGOUT | BOARD_HVS_DIGIN | BOARD_FLASH_MEMORY);

    //overwrite some pinmux...		overwrite PRU pin mux for GPIO4_17
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_CCDC1_DATA3),
         (CONTROL_CONF_MUXMODE(7)));
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_LCD_DATA6), (CONTROL_CONF_MUXMODE(7))); // gpio2_12
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_GPMC_AD13), (CONTROL_CONF_MUXMODE(7))); // gpio1_13
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_GPMC_AD14), (CONTROL_CONF_MUXMODE(7))); // gpio1_14
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_CCDC1_DATA5),
          (CONTROL_CONF_MUXMODE(7)));
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_GPMC_AD12),
          (CONTROL_CONF_MUXMODE(7)));
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_GPMC_AD15),
          (CONTROL_CONF_MUXMODE(7)));
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_CCDC1_DATA7),
          (CONTROL_CONF_MUXMODE(7)));
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_UART0_RTSN),
          (CONTROL_CONF_MUXMODE(7)));

    // BIOS tasks setup
	Task_Params_init(&taskParams);
 	taskParams.stackSize = 0x600;
	taskParams.priority = 2;

    taskParams.priority = 1;
 	Task_create (MPEncoderTask, &taskParams, NULL);
    Task_Params_init(&taskParams);

    CONSOLEUtilsInit();
    CONSOLEUtilsSetType(CONSOLE_UTILS_TYPE_UART);

   BIOS_start();
   return 0;
}

/**
 * @brief encoder task, which checks if any of the supported encoders
 * are connected.
 *
 * @param arg0 not used
 * @param arg1 not used
 */

void MPEncoderTask(UArg arg0, UArg arg1)
{
	unsigned int i=0, n=0;
	PRU_init();
	MPEncoderInit();
	do{
		mpe_protocol_type = mpe_no_protocol;
		switch(i)
		{
		case 0:
			MPEncoderBiSS();
			break;
		case 1:
			MPEncoderEnDat22();
			break;
		case 2:
			MPEncoderHiperfaceDSL();
			break;
		default:
			break;
		}
		i++;
		n++;
		if(i>=3)
		{
			i=0;
			if(n&0x1)
				CONSOLEUtilsPrintf("Testing for encoder *\r");
			else
				CONSOLEUtilsPrintf("Testing for encoder .\r");
		}
	}while(1);
}


