/*
* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
*
*    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include <stdio.h>
#include <stdlib.h>
#include "mpe_common.h"
#include "pruicss.h"
#include "hw_types.h"
#include "gpio.h"
#include "console_utils.h"
#include "osdrv_osal.h"
#include "osdrv_pruss.h"
#include "hw_icss.h"
#include "hw_icss_pru_ctrl.h"
#include "hw_icss_cfg.h"
#include "pruicss_v1.h"
#include "am437x.h"
#include "hw_control_am43xx.h"
#include "hw_cm_dpll.h"
#include "hw_cm_wkup.h"
#include "board_gpio.h"

PRUICSS_Handle handle, pruIcss0Handle;

/**
 * @brief performs generic initalization of the encoder adapter board hardware
 *
 *
 */
void MPEncoderInit(void)
{
	Board_enableGPIOClock(0xFF);

	// GPIO direction
	GPIOSetDirMode(SOC_GPIO2_REG, 26, GPIO_DIRECTION_OUTPUT);
	GPIOSetDirMode(SOC_GPIO2_REG, 12, GPIO_DIRECTION_OUTPUT);
	GPIOSetDirMode(SOC_GPIO1_REG, 13, GPIO_DIRECTION_OUTPUT);
	GPIOSetDirMode(SOC_GPIO1_REG, 14, GPIO_DIRECTION_OUTPUT);

    // Multi-protocol Encoder pinmuxing (EnDAT, BiSS, Hiperface DSL)
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MCASP0_AXR1), (CONTROL_CONF_MUXMODE(5)));	// pr0_pru0_gpo6
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MCASP0_AHCLKX), (CONTROL_CONF_MUXMODE(5)));	// pr0_pru0_gpo7
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MMC0_DAT3), (CONTROL_CONF_MUXMODE(5)));	// pr0_pru0_gpo8
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MMC0_DAT0), (1 << 18) | (1 << 16) | (CONTROL_CONF_MUXMODE(6)));	// pr0_pru0_gpo11
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MCASP0_ACLKX), (1 << 18) | (1 << 16) | (CONTROL_CONF_MUXMODE(6)));	//pr0_pru0_gpo0
    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_CONF_MCASP0_FSX), (CONTROL_CONF_MUXMODE(5)));	//pr0_pru0_gpo1
}

/**
 * @brief configures the PRU-ICSS to operate with 225MHz OCP clock
 *
 *
 */
void conf_225MHZ(void)
{
	unsigned int temp32;
	// set DPLL in bypass mode
	HW_WR_REG32(SOC_CM_WKUP_REG + PRCM_CM_CLKMODE_DPLL_DISP, 0x04);
	// wait for bypass to be set
	while(!(HW_RD_REG32(SOC_CM_WKUP_REG + PRCM_CM_IDLEST_DPLL_DISP) & (1<<8)));
	// configure clksel DPLL (24MHz / (N+1) * M / N2 = 24MHz / (23+1) * 900 / 4
	temp32 = HW_RD_REG32(SOC_CM_WKUP_REG + PRCM_CM_CLKSEL_DPLL_DISP) & (~0x7FFFF);
	temp32 |= (900<<8) | 23;
	HW_WR_REG32(SOC_CM_WKUP_REG + PRCM_CM_CLKSEL_DPLL_DISP, temp32);
	// configure CM_DIV_M2_DPLL_DISP
	HW_WR_REG32(SOC_CM_WKUP_REG + PRCM_CM_DIV_M2_DPLL_DISP, ((0xFFFFFFE0 | 4)));
	// lock DPLL and wait for confirmation
	HW_WR_REG32(SOC_CM_WKUP_REG + PRCM_CM_CLKMODE_DPLL_DISP, 0x07);
	while(!(HW_RD_REG32(SOC_CM_WKUP_REG + PRCM_CM_IDLEST_DPLL_DISP) & (1<<0)));
	// set OCP clock to source DPLL
	HW_WR_REG32(SOC_CM_DPLL_REG + PRCM_CM_DPLL_CLKSEL_ICSS_OCP_CLK, 0x01);
}

/**
 * @brief configures the PRU-ICSS to operate with 200MHz OCP clock
 *
 *
 */
void conf_200MHZ(void)
{
	// set OCP clock to source L3F clock
	HW_WR_REG32(SOC_CM_DPLL_REG + PRCM_CM_DPLL_CLKSEL_ICSS_OCP_CLK, 0x00);
}

/**
 * @brief initializes PRU-ICSS0 and PRU-ICSS1 subsystem.
 *
 *
 */
void PRU_init(void)
{
    handle = (PRUICSS_Handle)malloc(sizeof(PRUICSS_Config));
	handle->object = (PRUICSS_V1_Object*)malloc(sizeof(PRUICSS_V1_Object));
	handle->hwAttrs = (PRUICSS_HwAttrs*)malloc(sizeof(PRUICSS_HwAttrs));
	((PRUICSS_V1_Object*)handle->object)->instance = 1;

	pruIcss0Handle = (PRUICSS_Handle)malloc(sizeof(PRUICSS_Config));
	pruIcss0Handle->object = (PRUICSS_V1_Object*)malloc(sizeof(PRUICSS_V1_Object));
	pruIcss0Handle->hwAttrs = (PRUICSS_HwAttrs*)malloc(sizeof(PRUICSS_HwAttrs));
	((PRUICSS_V1_Object*)pruIcss0Handle->object)->instance = 0;

    PRUSSDRVInit(handle); /* ICSS_M instance 0 */
	PRUSSDRVInit(pruIcss0Handle);

	// Enable OCP master ports for ICSS0/1 and
	PRUICSS_enableOCPMasterAccess(pruIcss0Handle); // ICSS0
	PRUICSS_enableOCPMasterAccess(handle);	// ICSS1

	// Set in constant table C30 to shared RAM 0x40300000
	HWREG(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0) + HW_ICSS_PRU_CTRL_CTPPR1) = (0x40300000 & 0x00FFFF00) >> 8;
	// disable address offset
	HWREG(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_CFG + HW_ICSS_CFG_PMAO) = 0;

}

/**
 * @brief Set the C28/C28 contant table in PRU-ICSS
 *
 *
 */
void ConfigureConstantTable(void)
{
	// configure C28 to point to PRU_ICSS_CTRL and configure C29 to point to EDMA+0x1000
	//HW_WR_REG32(PRUSS_ICSS + ICSS_0 + PRU_ICSS_CTRL_CTPPR0, ((RD_MEM_32(PRUSS_ICSS + ICSS_0 + PRU_ICSS_CTRL_CTPPR0) & 0xffff0000) | (0x0220)));
	HW_WR_REG32(((PRUICSS_HwAttrs *)(pruIcss0Handle->hwAttrs))->baseAddr + PRU_ICSS_PRU_CTRL(0) +  HW_ICSS_PRU_CTRL_CTPPR0, ((0x0010<<16)| (0x0220)));
}
